
/*
 * I will use these macros to replace lwl/lwr/swl/swr instructions
 * Alan Liu
 * 2007/7/29
 */
#ifndef _LX_MACRO_H_
#define _LX_MACRO_H_
#ifdef CONFIG_CPU_LX45XXX
/*
 * load values, dont care about addr status
 * I use t5 as the temporary register to keep values
 * I use t6 as the temporary register to keep address
 * Alan Liu
 * 2007/7/29
 */
#define EXC_UNALIGNED_LD(value_reg,addr_reg,offset,handler)	\
        addiu   t6,addr_reg,offset;                     \
	move	value_reg,zero;				\
9:	lbu	t5,0(t6);				\
	sll	value_reg,8;                            \
	or	value_reg,t5,value_reg;			\
10:	lbu	t5,1(t6);				\
	sll	value_reg,8;                            \
	or	value_reg,t5,value_reg;			\
11:	lbu	t5,2(t6);				\
	sll	value_reg,8;                            \
	or	value_reg,t5,value_reg;			\
12:	lbu	t5,3(t6);				\
	sll	value_reg,8;                            \
	or	value_reg,t5,value_reg;			\
        .section __ex_table,"a";                        \
        PTR     9b, handler;                            \
        PTR     10b, handler;                           \
        PTR     11b, handler;                           \
        PTR     12b, handler;                           \
        .previous

 /*
  *
  * store values in value_reg to offset(addr_reg),dont care 
  * about the address
  *
  */
#define EXC_UNALIGNED_ST(value_reg,addr_reg,offset,handler)	\
	addiu   t6,addr_reg,offset;                     \
	andi    t5,value_reg,0xff;                      \
st9:	sb      t5, 3(t6);                              \
	srl     value_reg,8;                            \
	andi    t5,value_reg,0xff;                      \
st10:	sb      t5, 2(t6);                              \
	srl     value_reg,8;                            \
	andi    t5,value_reg,0xff;                      \
st11:	sb      t5, 1(t6);                              \
	srl     value_reg,8;                            \
	andi    t5,value_reg,0xff;                      \
st12:	sb      t5, 0(t6);                              \
	.section __ex_table,"a";                        \
	PTR     st9b, handler;                            \
	PTR     st10b, handler;                           \
	PTR     st11b, handler;                           \
	PTR     st12b, handler;                           \
	.previous

/*
 * I need take care that the addr_reg+offset may be aligned to 4 bytes 
 * boundary, So I need take care about this. Some times LDFIRST is used
 * to load data when src % 4 == 0.(I got it from the memcpy.S). 
 * use t7 to adjust left shift numbers...
 */ 
#define EXC_LDFIRST(value_reg,addr_reg,offset,handler)	\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	sll	t5,3;					\
	srl	t6,2;					\
	sll	t6,2;					\
8:	lw	value_reg,0(t6);			\
	sll	value_reg,t5;				\
	.section __ex_table,"a";                        \
	PTR     8b, handler;                           \
	.previous

#define EXC_LDREST(value_reg,addr_reg,offset,handler)	\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	li	t7,3;					\
	beq	t5,t7,9f;				\
	subu	t7,t7,t5;				\
	sll	t7,3;					\
	srl	t6,2;					\
	sll	t6,2;					\
8:	lw	t5,0(t6);				\
	srl	t5,t5,t7;				\
	or	value_reg,t5;				\
9:	nop;						\
	.section __ex_table,"a";                        \
	PTR     8b, handler;				\
	.previous
	
#define EXC_STFIRST(value_reg,addr_reg,offset,handler)		\
	move	t7,value_reg;				\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	beqz	t5,8f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,9f;					\
	srl	t7,8;					\
	subu	t5,t5,0x1;				\
	beqz	t5,10f;					\
	srl	t7,8;					\
	j	11f;					\
	srl	t7,8;					\
8:	sw	t7,0(t6);				\
	j	12f;					\
	 nop;						\
9:	sb	t7,2(t6);				\
	srl	t7,8;					\
10:	sb	t7,1(t6);				\
	srl	t7,8;					\
11:	sb	t7,0(t6);				\
12:	nop;						\
	.section __ex_table,"a";                        \
	PTR     8b, handler;                            \
	PTR     9b, handler;                            \
	PTR     10b, handler;                           \
	PTR     11b, handler;                           \
	.previous

#define EXC_STREST(value_reg,addr_reg,offset,handler)		\
	move	t7,value_reg;				\
	addiu   t6,addr_reg,offset;	                \
	xori	t5,t6,0x3;				\
	andi	t5,0x3;					\
	beqz	t5,12f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,9f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,10f;					\
	 addiu	t6,1;					\
	j	11f;					\
	 addiu	t6,1;					\
9:	sb	t7,0(t6);				\
	srl	t7,8;					\
10:	sb	t7,-1(t6);				\
	srl	t7,8;					\
11:	sb	t7,-2(t6);				\
12:	nop;						\
	.section __ex_table,"a";                        \
	PTR     9b, handler;                            \
	PTR     10b, handler;                           \
	PTR     11b, handler;                           \
	.previous

#define EXC_LDFIRST_NOHANDLER(value_reg,addr_reg,offset)\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	sll	t5,3;					\
	srl	t6,2;					\
	sll	t6,2;					\
8:	lw	value_reg,0(t6);			\
	sll	value_reg,t5;				

#define EXC_LDREST_NOHANDLER(value_reg,addr_reg,offset)		\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	li	t7,3;					\
	beq	t5,t7,9f;				\
	subu	t7,t7,t5;				\
	sll	t7,3;					\
	srl	t6,2;					\
	sll	t6,2;					\
8:	lw	t5,0(t6);				\
	srl	t5,t5,t7;				\
	or	value_reg,t5;				\
9:	nop;

#define EXC_STFIRST_NOHANDLER(value_reg,addr_reg,offset)		\
	move	t7,value_reg;				\
	addiu   t6,addr_reg,offset;	                \
	andi	t5,t6,0x3;				\
	beqz	t5,8f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,9f;					\
	srl	t7,8;					\
	subu	t5,t5,0x1;				\
	beqz	t5,10f;					\
	srl	t7,8;					\
	j	11f;					\
	srl	t7,8;					\
8:	sw	t7,0(t6);				\
	j	12f;					\
	 nop;						\
9:	sb	t7,2(t6);				\
	srl	t7,8;					\
10:	sb	t7,1(t6);				\
	srl	t7,8;					\
11:	sb	t7,0(t6);				\
12:	nop;						

#define EXC_STREST_NOHANDLER(value_reg,addr_reg,offset)		\
	move	t7,value_reg;				\
	addiu   t6,addr_reg,offset;	                \
	xori	t5,t6,0x3;				\
	andi	t5,0x3;					\
	beqz	t5,12f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,9f;					\
	 nop;						\
	subu	t5,t5,0x1;				\
	beqz	t5,10f;					\
	 addiu	t6,1;					\
	j	11f;					\
	 addiu	t6,1;					\
9:	sb	t7,0(t6);				\
	srl	t7,8;					\
10:	sb	t7,-1(t6);				\
	srl	t7,8;					\
11:	sb	t7,-2(t6);				\
12:	nop;

#endif /* #ifdef CONFIG_CPU_LX45XXX */

#endif
