/*
 * TTI(Shanghai),Co.Ltd
 * Alan Liu(Liu Hongming)
 * 09/09/2003
 *
 */

#include <asm/asm.h>
#include <asm/asmmacro.h>
#include <asm/cacheops.h>
#include <asm/irqflags.h>
#include <asm/regdef.h>
#include <asm/fpregdef.h>
#include <asm/mipsregs.h>
#include <asm/stackframe.h>
#include <asm/addrspace.h>
#include <asm/war.h>
#include <asm/page.h>

/**
 * This functions within are written in assembly.
 */
//#define __ASSEMBLY__
#include <asm/trihidtv/trihidtv_irq.h>
#include <asm/trihidtv/trihidtv_defs.h>
#include <asm/trihidtv/trihidtv_vectIntr.h>

		.text
		.set	noreorder
/**
 * @file int-handler.S
 *
 * This file contains the assembly language interrupt dispatchers for the 
 * trident HiDTV SOC.  There are two routines provided. The first, lx_handle_int, 
 * is registered as the MIPS interrupt exception handler, therefore, handles 
 * the MIPS style interrupts. The second handler lx_vec_handle_int, handles 
 * the Lexra vectored interrupts. Each of the vectored interrupt vectors have
 * been coded to jump to this routine. This allows the different types of 
 * interrupts to be handled using the same functions in the higher level code.
 *
 * We follow the model in the Indy interrupt code by David Miller, where he
 * says: a lot of complication here is taken away because:
 *
 * 1) We handle one interrupt and return, sitting in a loop
 *    and moving across all the pending IRQ bits in the cause
 *    register is _NOT_ the answer, the common case is one
 *    pending IRQ so optimize in that direction.
 *
 * 2) We need not check against bits in the status register
 *    IRQ mask, that would make this routine slow as hell.
 *
 * 3) Linux only thinks in terms of all IRQs on or all IRQs
 *    off, nothing in between like BSD spl() brain-damage.
 *
 * LX4xxx/5xxx interrupt mapping:
 *@verbatim
 	MIPS IRQ	Source
 	_________	________
 		0	Software (ignored)
 		1	Software (ignored)
 		2	Reserved	
 		3	Reserved	
 		4	Reserved	
 		5	PC_HIDTV_INT
 		6	DM_INT_INT
 		7	VIDEO_INTN	

		8	EXT_INT2
		9	EXT_INT1
		10	INT0_
		11	INT_BF_SI
		12	INT_BF_NOSI
		13	AUDIO_INTN
		14	VBS_INT0
		15	CPU_NMI
  
  We handle the IRQ according to _our_ priority. The priority of 
  an interrupt may be changed by changing its location in its mask 
  and irq_nr tables through modification of the initialization function
  trihidtv_init_int(). 
  The default priority is:
 
Highest ----  CPU_NMI	0       //From ITE south bridge block
		VBS_INT0        1       //From DEMUX_QUARC block
		AUDIO_INTN      2       //From AUDIO block
		INT_BF_NOSI     3       //From DEMUX_QUARC
		INT_BF_SI       4       //From DEMUX_QUARC
		INT0            5       //From ITE south bridge block
		EXT_INT1        6       //From HiDTV pin
		EXT_INT2        7       //From HiDTV pin. It outputs interrupt when
                               	//in slaver mode. It is used as input interrupt
                               	//when in master mode.
		VIDEO_INTN      8       //From VIDEO block.
		DMINT_INT       9       //From DEMUX_QUARC block.
		PC_HIDTV_INT    10      //From ITE block. It only is used as
								//PC issue request to CPU when in pc mode.
		SW1             14      //Software Interrupt 1
Lowest  ----  SW0		15      //Software Interrupt 0

  0~~~7 is vectored interrupt,and 8,9,10 is mips standard interrupts.

 @endverbatim
 * The highest priority pending interrupt is handled first
 * then we just return, if multiple IRQs are pending then we will just take
 * another exception, big deal.
 *
 */

/*
 * The interrupt handler for Lexra.
 * The highest priority pending interrupt is handled first
 * then we just return, if multiple IRQs are pending then we will just take
 * another exception.
 * Spurious interrupts are ignored.
 */
/*
 * I change the origianl handler from 2.4, merged with handle_int from genex.S
 * So Maybe I need check it later...
 * Alan Liu
 * 2007/7/18
 */
NESTED(lx_handle_int, PT_SIZE, sp)
#ifdef CONFIG_TRACE_IRQFLAGS
	/*
	 * Check to see if the interrupted code has just disabled
	 * interrupts and ignore this interrupt for now if so.
	 *
	 * local_irq_disable() disables interrupts and then calls
	 * trace_hardirqs_off() to track the state. If an interrupt is taken
	 * after interrupts are disabled but before the state is updated
	 * it will appear to restore_all that it is incorrectly returning with
	 * interrupts disabled
	 */
	.set	push
	.set	noat
	mfc0	k0, CP0_STATUS
	/*
 	 * Add LX45XXX support.
 	 * Alan Liu
 	 * 2007/7/17
 	 */
	/* 
	 * #if defined(CONFIG_CPU_R3000) || defined(CONFIG_CPU_TX39XX) 
	 */
#if defined(CONFIG_CPU_R3000) || defined(CONFIG_CPU_TX39XX) || defined(CONFIG_CPU_LX45XXX)
	and	k0, ST0_IEP
	bnez	k0, 1f

	mfc0	k0, EP0_EPC
	.set	noreorder
	j	k0
	rfe
#else /* #ifdef CONFIG_TRACE_IRQFLAGS */
	and	k0, ST0_IE
	bnez	k0, 1f

	eret
#endif
1:
	.set pop
#endif /* #ifdef CONFIG_TRACE_IRQFLAGS */

	SAVE_ALL
	CLI
	TRACE_IRQS_OFF

#ifdef CONFIG_CPU_LX45XXX
	.set	noreorder
	/*
	 * Get pending Interrupts
	 */
	mfc0	t0,CP0_CAUSE		# get pending interrupts
	mfc0	t2,CP0_STATUS
	la		t1,cpu_mask_tbl
	andi	t0,0xff00
	and		t0,t2			# isolate allowed ones

	/* 
	 * detection of spurious interrupts 
	 */
	beq	t0,zero,return_	
	 nop			        # delay slot
	
	/*
	 * Find irq with highest priority
	 */
1:	lw		t2,(t1)
	move	t3,t0
	and		t3,t2
	beq		t3,zero,1b
	 addu	t1,PTRSIZE		# delay slot

	/*
	 * Do the low-level stuff
	 */
	#the value of a0 should be the
	#irq number used as the first parameter of do_IRQ.
	lw	a0,%lo(cpu_irq_nr-cpu_mask_tbl-PTRSIZE)(t1) 
	 nop
	bgez	a0, handle_it		# irq_nr >= 0?
					# irq_nr < 0: just exit
	 nop
	j	return_
	 nop				# delay slot

	/*
	 * Should call trihidtv_hw0_irqdispatch() first 
	 * before calling do_IRQ().
	 * Alan Liu
	 */		
handle_it:	
	LONG_L	s0, TI_REGS($28)
	LONG_S	sp, TI_REGS($28)
	jal	trihidtv_hw0_irqdispatch
	 move	a1,sp

return_:	
	j	ret_from_irq
	  nop
#else
	LONG_L	s0, TI_REGS($28)
	LONG_S	sp, TI_REGS($28)
	PTR_LA	ra, ret_from_irq
	j	plat_irq_dispatch
#endif
	END(lx_handle_int)

/**
 * This is the interrupt handler for the Lexra vectored interrupts. Each of
 * the vectors have been setup to jump to this routine. Similar in operation
 * to the MIPS interrupt handler lx_handle_int().
 */
#ifdef DOC_GENERATE
asmlinkage void lx_vec_handle_int(void){};
#else
		NESTED(lx_vec_handle_int, PT_SIZE, ra)
		.set	noat
		SAVE_ALL
		CLI				# TEST: interrupts should be off
		TRACE_IRQS_OFF
		.set	at
		.set	noreorder

		/**
		 * Get pending Interrupts using
		 * hand compiled Lexra specific instructions
		 */
		/* MFLXC0 t0,LX_ECAUSE */
		.word	(MFLXCO | (8<<16) | (LX_ECAUSE<<11)) 
		 nop
		 nop
		/* MFLXC0 t2,LX_ECAUSE */
		.word	(MFLXCO | (10<<16) | (LX_ESTATUS<<11)) 
		 nop
		 nop
		la	t1,lx_mask_tbl
		and	t0,t2			# isolate allowed ones

		/* 
		 *  detection of spurious interrupts 
		 */
		beq	t0,zero,lx_return_
		 nop			        # delay slot
		
		/**
		 * Find irq with highest priority, if none found just return.
		 */

1:		lw	t2,(t1)
		move	t3,t0
		and	t3,t2
		beq	t3,zero,1b
		 addu	t1,PTRSIZE		# delay slot

		/*
		 * Do the low-level stuff
		 */
		/*
		 * equal to: lw	a0, PTRSIZE*8(t1) 
		 */
		lw	a0,%lo(lx_irq_nr-lx_mask_tbl-PTRSIZE)(t1)
		 nop
		bgez	a0, lx_handle_it		# irq_nr >= 0?
							# irq_nr < 0: just exit
		 nop
		
		j	lx_return_
		 nop				# delay slot

		/*
		 * Should call trihidtv_hw0_irqdispatch() first 
		 * before calling do_IRQ().
		 * Alan Liu
		 */		
lx_handle_it:	
		LONG_L  s0, TI_REGS($28)
        	LONG_S  sp, TI_REGS($28)
		jal	trihidtv_hw0_irqdispatch
		 move	a1,sp
		
lx_return_:	
		j	ret_from_irq
		 nop

		END(lx_vec_handle_int)
#endif

/**
 * The function for masking the Lexra vectored interrupts using Lexra specific 
 * opcodes.  
 * @param mask Compliment of mask passed in through register a0.
 */
#ifdef DOC_GENERATE
inline void mask_lx_interrupt(unsigned int mask){};
#else
		LEAF(mask_lx_interrupt) #compliment of mask in a0
		
		move	t0, a0
		.word	(MFLXCO | (9<<16) | (LX_ESTATUS<<11)) #get status reg t1
		nop
		 nop
		and	t0, t1	# and the register with the mask complement
		.word	(MTLXCO | (8<<16) | (LX_ESTATUS<<11)) # copy back status
		nop
		 nop
		j	ra
		 nop
		
		END(mask_lx_interrupt)
#endif

/** 
 * The function used for unmasking the Lexra vectored interrupts using Lexra 
 * specific opcodes. 
 * @param mask Mask passed in through register a0.
 */
#ifdef DOC_GENERATE
inline void unmask_lx_interrupt(unsigned int mask){};
#else
		LEAF(unmask_lx_interrupt)
		
		move	t0, a0
		.word	(MFLXCO | (9<<16) | (LX_ESTATUS<<11)) #get status reg t1
		 nop
		 nop
		or	t0, t1, t0 # and the register with the mask complement
		.word	(MTLXCO | (8<<16) | (LX_ESTATUS<<11)) # copy back status
		 nop
		 nop
		j	ra
		 nop
		 	
		END(unmask_lx_interrupt)
#endif

/** 
 * The function used for read the Lexra vectored interrupts EINTVEC register 
 * usingLexra specific opcodes.  
 * @param reg Pointer to the location to store the value in the cause register.
 *
 * Alan Liu
 * 09/12/2003
 */
#ifdef DOC_GENERATE
asmlinkage void read_lx_intvec(unsigned int* reg){};
#else
		LEAF(read_lx_intvec)

		.word (MFLXCO | (8<<16) | (LX_INTVEC<<11))
		 nop
		 nop
		sw	t0, 0(a0)
		 nop
		j 	ra
		 nop

		END(read_lx_intvec)
#endif

/** 
 * The function used for write value into Lexra vectored interrupts EINTVEC 
 * register using Lexra specific opcodes.  
 * @param reg Pointer to the location to store the value in the cause register.
 *
 * Alan Liu
 * 09/12/2003
 */
#ifdef DOC_GENERATE
asmlinkage void write_lx_intvec(unsigned int reg){};
#else
		LEAF(write_lx_intvec)

		.word (MTLXCO | (4<<16) | (LX_INTVEC<<11)) 
		 nop
		 nop
		j 	ra
		 nop

		END(write_lx_intvec)
#endif

/** 
 * The function used for read value from Lexra vectored interrupts ECAUSE register 
 * using Lexra specific opcodes.  
 * @param reg Pointer to the location to store the value in the cause register.
 */
#ifdef DOC_GENERATE
asmlinkage void read_lx_cause(unsigned int* reg){};
#else
		LEAF(read_lx_cause)

		.word (MFLXCO | (8<<16) | (LX_ECAUSE<<11))
		 nop
		 nop
		sw	t0, 0(a0)
		 nop
		j 	ra
		 nop

		END(read_lx_cause)
#endif

/** 
 * A function used to read the Lexra processor revision register.
 * @return The value in the revision register.
 */
#ifdef DOC_GENERATE
asmlinkage unsigned int read_lx_revision(void){};
#else
		LEAF(read_lx_revision)

		.word (MFLXCO | (2<<16) | (3<<11))
		 nop
		 nop
		j 	ra
		 nop

		END(read_lx_revision)
#endif

/** 
 * The function used for reading the Lexra vectored interrupt status register 
 * using Lexra specific opcodes.
 * @param reg Pointer to the location to store the value in the status register.
 */
#ifdef DOC_GENERATE
asmlinkage void read_lx_status(unsigned int* reg){};
#else
		LEAF(read_lx_status)

		.word (MFLXCO | (8<<16) | (LX_ESTATUS<<11))
		 nop
		 nop
		sw	t0, 0(a0)
		 nop
		j 	ra
		 nop

		END(read_lx_status)
#endif
#ifdef CONFIG_CPU_LX45XXX
/*
 * Lexra specific vectored interrupt handlers
 * So EINTVEC should be set to the start address of lx_int_vec8!
 * You could see this in trihidtv/lx.c
 * lx_vec_handle_int is in int-handler.S
 * align it to 2^6 to make sure bit 5~~0 is zero.
 * Alan Liu
 * 2007/7/18
 */
		.align 6
		NESTED(lx_int_vec8, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec8)

		NESTED(lx_int_vec9, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec9)	

		NESTED(lx_int_vec10, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec10)	

		NESTED(lx_int_vec11, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec11)	

		NESTED(lx_int_vec12, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec12)	

		NESTED(lx_int_vec13, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec13)	

		NESTED(lx_int_vec14, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec14)	

		NESTED(lx_int_vec15, 0, sp)
		j	lx_vec_handle_int
		nop
		END(lx_int_vec15)	
#endif		

/**
 * The cpu_mask_tbl contains the interrupt masks for the interrupts in 
 * the corresponding positions of the cpu_irq_nr table. It is initialised 
 * in the funtion trihidtv_init_int() found in the file lx.c.
 */
#ifdef DOC_GENERATE
unsigned long cpu_mask_tbl[8];
#else
		.data
		EXPORT(cpu_mask_tbl)
cpu_mask_tbl:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000		# these two are unlikely 
		.word	0x00000000		# to be used
		.word	0x0000ff00		# End of list
#endif

/**
 *  This array maps each of the interrupts to a number enabling configuation
 *  of interrupt priority.
 */
#ifdef DOC_GENERATE
unsigned long cpu_irq_nr[8];
#else
		EXPORT(cpu_irq_nr)
cpu_irq_nr:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000		# these two are unlikely 
		.word	0x00000000		# to be used
		.word	0x00ffffff		# End of list
#endif

/**
 * The lx_mask_tbl contains the interrupt masks for the interrupts in the 
 * corresponding positions of the lx_irq_nr table. It is initialised in the 
 * funtion trihidtv_init_int() found in the file lx.c.
 */
#ifdef DOC_GENERATE
unsigned long lx_mask_tbl[8];
#else
		EXPORT(lx_mask_tbl)
lx_mask_tbl:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0xffffffff			# EOL
#endif

/**
 * This array maps each of the interrupts to a number enabling configuation
 * of interrupt priority.
 */
#ifdef DOC_GENERATE
unsigned long lx_irq_nr[8];
#else
		EXPORT(lx_irq_nr)
lx_irq_nr:
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0x00000000
		.word	0xffffffff		# EOL
#endif

