/**
 *
 * This file contains code to interface generic Linux interrupt handling
 * functions in arch/mips/kernel/irq.c. Additional 
 * code added to implement the Lexra vectored interrupts.
 *
 * SHOULD PAY much attention to the issues of Endian!!!!!
 *
 * Alan Liu 	alanliu@trident.com.cn
 * TTI Shanghai
 * 09/10/2003
 */

#include <linux/errno.h>
#include <linux/init.h>
#include <linux/kernel_stat.h>
#include <linux/signal.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
//#include <linux/trace.h>
#include <linux/timex.h>
//#include <linux/malloc.h>
#include <linux/random.h>
#include <linux/irq.h>

#include <asm/bitops.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/mipsregs.h>
#include <asm/system.h>

#include <asm/trihidtv/trihidtv.h>
#include <asm/trihidtv/trihidtv_int.h>
#include <asm/trihidtv/trihidtv_irq.h>
#include <asm/trihidtv/trihidtv_defs.h>
#include <asm/trihidtv/trihidtv_addrspace.h>

/**
 * External function defined in int-handler.S. This is an assembly language
 * routine used to mask the Lexra vectored interrupts using Lexra specific
 * instructions.
 */
extern void mask_lx_interrupt(unsigned int mask);

/**
 * External function defined in int-handler.S. This is an assembly language
 * routine used to unmask the Lexra vectored interrupts using Lexra specific
 * instructions.
 */
extern void unmask_lx_interrupt(unsigned int mask);

extern int pci_devices_count; 
extern void lx_irq_setup(void);
extern void init_generic_irq(void);
//extern void do_IRQ(int,struct pt_regs *);	

#ifndef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS

struct ite_intc_regs volatile *ite_hw0_icregs
        = (struct ite_intc_regs volatile *)(KSEG1ADDR(TRIHIDTV_PCI_IO_BASE + TRIHIDTV_INTC_BASE));

void disable_ite_irq(unsigned int irq_nr)
{
	unsigned short tmp;

	if ( (irq_nr >= TRIHIDTV_LPC_IRQ_BASE) && (irq_nr <= TRIHIDTV_LPC_IRQ_BASE + 15)) {
		/* LPC interrupt */
		panic("disable_ite_irq: there are no lpc irq: %d in our chip", irq_nr);
	}
	else if ( (irq_nr >= TRIHIDTV_LB_IRQ_BASE) && (irq_nr <= TRIHIDTV_LB_IRQ_BASE + 15)) {
		/* Local Bus interrupt */
		tmp = ReadRegHWord(&ite_hw0_icregs->lb_mask);
		WriteRegHWord(&ite_hw0_icregs->lb_mask,tmp|(1 << (irq_nr - TRIHIDTV_LB_IRQ_BASE)));
	}
	else if ( (irq_nr >= TRIHIDTV_PCI_DEV_IRQ_BASE) && (irq_nr <= TRIHIDTV_PCI_DEV_IRQ_BASE + 15)) {
		/* PCI and other interrupts */
		tmp = ReadRegHWord(&ite_hw0_icregs->pci_mask);
		WriteRegHWord(&ite_hw0_icregs->pci_mask,tmp|(1 << (irq_nr - TRIHIDTV_PCI_DEV_IRQ_BASE)));
	}
	else if ( (irq_nr >= TRIHIDTV_NMI_IRQ_BASE) && (irq_nr <= TRIHIDTV_NMI_IRQ_BASE + 15)) {
		/* NMI interrupts */
		tmp = ReadRegHWord(&ite_hw0_icregs->nmi_mask);
		WriteRegHWord(&ite_hw0_icregs->nmi_mask,tmp|(1 << (irq_nr - TRIHIDTV_NMI_IRQ_BASE)));
	}
	else {
		panic("disable_ite_irq: bad irq %d", irq_nr);
	}
}

void enable_ite_irq(unsigned int irq_nr)
{
	unsigned short tmp;
	if ( (irq_nr >= TRIHIDTV_LPC_IRQ_BASE) && (irq_nr <= TRIHIDTV_LPC_IRQ_BASE + 15)) {
		/* LPC interrupt */
		panic("enable_ite_irq: there are no lpc irq: %d in our chip", irq_nr);
	}
	else if ( (irq_nr >= TRIHIDTV_LB_IRQ_BASE) && (irq_nr <= TRIHIDTV_LB_IRQ_BASE + 15)) {
		/* Local Bus interrupt */
		tmp = ReadRegHWord(&ite_hw0_icregs->lb_mask);
		WriteRegHWord(&ite_hw0_icregs->lb_mask,tmp & ~(1 << (irq_nr - TRIHIDTV_LB_IRQ_BASE)));
	}
	else if ( (irq_nr >= TRIHIDTV_PCI_DEV_IRQ_BASE) && (irq_nr <= TRIHIDTV_PCI_DEV_IRQ_BASE + 15)) {
		/* PCI and other interrupts */
		tmp = ReadRegHWord(&ite_hw0_icregs->pci_mask);
		WriteRegHWord(&ite_hw0_icregs->pci_mask,tmp & ~(1 << (irq_nr - TRIHIDTV_PCI_DEV_IRQ_BASE)));
	}
	else if ( (irq_nr >= TRIHIDTV_NMI_IRQ_BASE) && (irq_nr <= TRIHIDTV_NMI_IRQ_BASE + 15)) {
		/* NMI interrupts */
		tmp = ReadRegHWord(&ite_hw0_icregs->nmi_mask);
		WriteRegHWord(&ite_hw0_icregs->nmi_mask,tmp & ~(1 << (irq_nr - TRIHIDTV_NMI_IRQ_BASE)));
	}
	else {
		panic("enable_ite_irq: bad irq %d", irq_nr);
	}
}
#endif /* CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS */

/**
 * Internal function to mask an interrupt. Will mask any of the three
 * types of Lexra interrupts: standard MIPS or Lexra vectored.
 * @param irq_nr The interrupt number to be masked.
 */
/*
 * since irq_nr in our chips is above than 16,
 * but the original FPGA board has only 16 interrupts,
 * so irq_nr should be classified deeper...
 * Alan Liu
 * 09/11/2003
 */
static inline void mask_irq(unsigned int irq_nr)
{		

	if(IS_CPU_IRQ(irq_nr)){		//0~~~7
		//clear_cp0_status(lx_interrupt[irq_nr].cpu_mask);
		clear_c0_status(lx_interrupt[irq_nr].cpu_mask);
	}
	else if(IS_VECTOR_IRQ(irq_nr)){//8~~~15
		mask_lx_interrupt(~lx_interrupt[irq_nr].LX_mask);
	}
#ifndef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
	else if(IS_NMI_IRQ(irq_nr)){
		disable_ite_irq(irq_nr);
	}
	else if(IS_INT0_IRQ(irq_nr)) {
		disable_ite_irq(irq_nr);
	}
#endif
	else
		panic("mask_irq: bad irq number %d",irq_nr);
}

/**
 * Internal function to unmask an interrupt. Will unmask any of the three
 * types of Lexra interrupts: standard MIPS or Lexra vectored.
 * @param irq_nr The interrupt number to be unmasked.
 */
/*
 * since irq_nr in our chips is above than 16,
 * but the original FPGA board has only 16 interrupts,
 * so irq_nr should be classified deeper...
 * Alan Liu
 * 09/11/2003
 */
static inline void unmask_irq(unsigned int irq_nr)
{	
	if(IS_CPU_IRQ(irq_nr)){		//0~~~7
		//set_cp0_status(lx_interrupt[irq_nr].cpu_mask);
		set_c0_status(lx_interrupt[irq_nr].cpu_mask);
	}
	else if(IS_VECTOR_IRQ(irq_nr)){//8~~~15
		unmask_lx_interrupt(lx_interrupt[irq_nr].LX_mask);
	}
#ifndef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
	else if(IS_NMI_IRQ(irq_nr)){
		enable_ite_irq(irq_nr);
	}
	else if(IS_INT0_IRQ(irq_nr)) {
		enable_ite_irq(irq_nr);
	}
#endif
	else
		panic("unmask_irq: bad irq number %d",irq_nr);
}


/**
 * Following four functions are needed by arch/mips/kernel/irq.c
 */
unsigned int startup_irq(unsigned int irq_nr)
{
	unmask_irq(irq_nr);
	return 0; 
}


static void shutdown_irq(unsigned int irq_nr)
{
	mask_irq(irq_nr);
}

/*
 *
 */
static inline void ack_level_irq(unsigned int irq_nr)
{
        mask_irq(irq_nr);
}


static inline void end_irq(unsigned int irq_nr)
{
	if (!(irq_desc[irq_nr].status & (IRQ_DISABLED|IRQ_INPROGRESS)))
		unmask_irq(irq_nr); 
	else {
		irq_desc_t *desc = irq_desc + irq_nr;
		//dump_desc_t(desc);
	}
}

static struct irq_chip trihidtv_irq_chip = {
	.name = "HiDTV IRQ CHIP",
	.startup = startup_irq,
	.shutdown = shutdown_irq,
	.unmask = unmask_irq,
	.mask = mask_irq,
	.ack = ack_level_irq,
	.end = end_irq,
	NULL
};

#ifndef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
/*
 * dispatch accoring to cpuirq
 * if cpuirq == 10(INT0),should dispatch deeper
 * if cpuirq == 15(NMI),should dispatch deeper
 * else	call do_IRQ directly.
 * Alan Liu
 * 09/09/2003
 * For AV mips, no need to call this function
 */
void trihidtv_hw0_irqdispatch(int cpuirq,struct pt_regs *regs)
{
	int irq;
	unsigned short intstatus = 0, status = 0;

	//if cpuirq is not NMI(15) or INT0(10)
	if(cpuirq != CPU_NMI_PIN && cpuirq != INT0_PIN){
		//do_IRQ(cpuirq,regs);	
		do_IRQ(cpuirq);	
		return;
	}

	//get Interrupt type according to INTC_TYPER
	//intstatus = ite_hw0_icregs->intstatus;
	intstatus = ReadRegHWord(&ite_hw0_icregs->intstatus);
	if (intstatus & INTC_TYPER_NMI) {
		/* NMI interrupt */
		irq = 0;
		//status |= ite_hw0_icregs->nmi_req;
		status |= ReadRegHWord(&ite_hw0_icregs->nmi_req);
		while (!(status & 0x1)) {
			irq++;
			status >>= 1;
		}
		irq += TRIHIDTV_NMI_IRQ_BASE;
		//printk("nmi int %d\n", irq);
	}
	else if (intstatus & INTC_TYPER_PCI) {
		/* PCI interrupt */
		irq = 0;
		//status |= ite_hw0_icregs->pci_req;
		status |= ReadRegHWord(&ite_hw0_icregs->pci_req);
		while (!(status & 0x1)) {
			irq++;
			status >>= 1;
		}
		irq += TRIHIDTV_PCI_DEV_IRQ_BASE;
		//printk("pci int %d\n", irq);
	}
	else if (intstatus & INTC_TYPER_LBD) {
		/* Local Bus interrupt */
		irq = 0;
		//status |= ite_hw0_icregs->lb_req;
		status |= ReadRegHWord(&ite_hw0_icregs->lb_req);
		while (!(status & 0x1)) {
			irq++;
			status >>= 1;
		}
		irq += TRIHIDTV_LB_IRQ_BASE;
		//printk("lb int %d\n", irq);
	}
	else if (intstatus & INTC_TYPER_LPC) {
		/* LPC interrupt */
		/* Since some lpc interrupts are edge triggered,
		 * we could lose an interrupt this way because
		 * we acknowledge all ints at onces. Revisit.
		 */
		panic("Why we Got LPC interrupt??!!");
		/*
		status |= ite_hw0_icregs->lpc_req;
		ite_hw0_icregs->lpc_req = 0; //acknowledge ints
		irq = 0;
		while (!(status & 0x1)) {
			irq++;
			status >>= 1;
		}
		irq += TRIHIDTV_LPC_IRQ_BASE;
		//printk("LPC int %d\n", irq);
		*/
	}
	else {
		return;
	}
	//do_IRQ(irq, regs);
	do_IRQ(irq);	
}
void
ite_intc_reset()
{

	/* mask all ite interrupts */
	WriteRegHWord(&ite_hw0_icregs->lb_mask,0xffff);
	WriteRegHWord(&ite_hw0_icregs->lpc_mask,0xffff);
	WriteRegHWord(&ite_hw0_icregs->pci_mask,0xffff);
	WriteRegHWord(&ite_hw0_icregs->nmi_mask,0xffff);

	/* clear ite request register */
	WriteRegHWord(&ite_hw0_icregs->lb_req,0);
	WriteRegHWord(&ite_hw0_icregs->lpc_req,0);
	WriteRegHWord(&ite_hw0_icregs->pci_req,0);
	WriteRegHWord(&ite_hw0_icregs->nmi_req,0);

	WriteRegHWord(&ite_hw0_icregs->lb_trigger,0);
	WriteRegHWord(&ite_hw0_icregs->lpc_trigger,0);
	WriteRegHWord(&ite_hw0_icregs->pci_trigger,0);
	WriteRegHWord(&ite_hw0_icregs->nmi_trigger,0);
	
	/* make all ite interrupts low level active */

	/*
	 * Be careful,LBD are all high active,and ADC0 is broken...
	 * set lb_level to ffff
	 * With help from Wangdong and Lily
	 */
	WriteRegHWord(&ite_hw0_icregs->lb_level,0xffff);
	WriteRegHWord(&ite_hw0_icregs->lpc_level,0);
	WriteRegHWord(&ite_hw0_icregs->pci_level,0);
	/*
	 * Be Careful here!
	 * When set all level to 0,then nmi_req will report 0xffec,
	 * The default value is 0xffec.
	 * so here 0xffec.
	 */
	WriteRegHWord(&ite_hw0_icregs->nmi_level,0xffec);

	/*
	 * We redirect timer2 to INT0!then its mask register could work!!!
	 *
	 */
	WriteRegHWord(&ite_hw0_icregs->nmi_redir,0x8000);
	WriteRegHWord(&ite_hw0_icregs->intstatus,0);
}
#endif /* CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS */

/**
 * Initial setup of interrupts. Called by start_kernel(). Only
 * function is to  call the function irq_setup().
 *
 * Alan Liu
 * 09/11/2003
 */
/*
 * Re-port it for linux 2.6.
 * 2007/7/17
 */
//void __init init_IRQ(void)
void __init arch_init_irq(void)
{
        int i;

#if 0
	/*
	 * irq_desc[NR_IRQS] is defined in arch/mips/kernel/irq.c
	 */
	memset(irq_desc, 0, sizeof(irq_desc));
#endif
	/*
	 * lx_irq_setup() is defined in arch/mips/trihidtv/lx.c
	 * it will call board specific function:  trihidtv_init_int()
	 * Alan Liu
	 * 09/12/2003
	 */
	lx_irq_setup();

	/*
	 * Init INTC
	 */
	ite_intc_reset();

	/* 
	 * 8~15 should be enabled on cpu(ESTATUS register);
	 * We should set ESTATUS,to make 15(NMI),10(INT0) is enabled in CPU's ESTATUS...
	 * Note: since our valid irq_nr is 0~~7 for standard mips interrupts,
	 * 	and others(except 10,15) for outer interrupts ,so 10,15
	 * 	shoud startup here,no request_req() will startup them(10,15)
	 * 	however,0~~7,8,9,11,12,13,14,16,17,... will be called by specific driver,and
	 * 	startup_irq will be call automatically in request_irq(),so we should call
	 * 	startup_irq(10),startup_irq(15) here manually.
	 *
	 * startup_irq() is defined in arch/mips/trihidtv/irq.c
	 *
	 */
#ifdef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
	/*
	 * AV MIPS Needs no INTC.
	 * So we should not startup 15,10 here.
	 * Alan Liu
	 */
#else
	startup_irq(CPU_NMI_PIN);
	startup_irq(INT0_PIN);
#endif

	for(i=0; i < NR_IRQS; i++) { 
		/* 
		 * for MIPS interrupts
		 * and for Lexra vectored interrupts 
		 * See include/asm/trihidtv/
		 * Modified by Alan Liu
		 * 09/11/2003
		 */
		//irq_desc[i].handler = &level_irq_type;
		/*
		 * I use this function to set the irq chip
		 * Alan Liu
		 * 2007/7/18
		 */
		set_irq_chip(i,&trihidtv_irq_chip);
	}
}

#if 1
void show_vectored_irqs_status(void)
{
	printk("intc lb_req: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->lb_req),__FILE__,__LINE__);
	printk("intc pci_req: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->pci_req),__FILE__,__LINE__);
	printk("intc nmi_req: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->nmi_req),__FILE__,__LINE__);

	printk("intc lb_mask: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->lb_mask),__FILE__,__LINE__);
	printk("intc pci_mask: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->pci_mask),__FILE__,__LINE__);
	printk("intc nmi_mask: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->nmi_mask),__FILE__,__LINE__);

	printk("intc lb_level: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->lb_level),__FILE__,__LINE__);
	printk("intc pci_level: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->pci_level),__FILE__,__LINE__);
	printk("intc nmi_level: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->nmi_level),__FILE__,__LINE__);

	printk("intc lb_trigger: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->lb_trigger),__FILE__,__LINE__);
	printk("intc pci_trigger: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->pci_trigger),__FILE__,__LINE__);
	printk("intc nmi_trigger: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->nmi_trigger),__FILE__,__LINE__);

	printk("intc typer: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->intstatus),__FILE__,__LINE__);
	printk("intc redirection: 0x%04x [%s:%d]\n",ReadRegHWord(&ite_hw0_icregs->nmi_redir),__FILE__,__LINE__);
}
#endif
