/*
 * TTI Shanghai
 * 09/10/2003
 * Alan Liu alanliu@trident.com.cn
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/mm.h>
#include <linux/swap.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/timex.h>
#include <linux/kernel_stat.h>
#include <asm/bootinfo.h>
#include <asm/page.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/processor.h>
#include <asm/ptrace.h>
#include <asm/reboot.h>
#include <asm/mc146818rtc.h>
#include <linux/version.h>
#include <linux/bootmem.h>
#include <asm/irq.h>

#include <linux/mc146818rtc.h>
#include <asm/trihidtv/trihidtv.h>
#include <asm/trihidtv/trihidtv_irq.h>
#include <asm/trihidtv/trihidtv_defs.h>
#include <asm/trihidtv/trihidtv_addrspace.h>

/* This is for machines which generate the exact clock. */
//#define USECS_PER_JIFFY (1000000/HZ)

/* usecs per counter cycle, shifted to left by 32 bits */
//static unsigned int sll32_usecs_per_cycle = 0;

/* Cycle counter value at the previous timer interrupt.. */
//static unsigned int timerlo;

/* the function pointer to one of the gettimeoffset funcs*/
//extern unsigned long (*do_gettimeoffset) (void);

extern unsigned int mips_counter_frequency;

/**
 * Function stub. Function not required for Lexra implementation of Linux.
 */
/*
void
page_is_ram(void)
{
};
*/
/**
 * Function stub. Function not required for Lexra implementation of Linux.
 */
void
prom_free_prom_memory(void)
{
};

/**
 * A structure to store parameters to be passed from the boot loader
 * to the kernel.
 */
//struct bootParams lxBootParams;

/**
 * Macro to get address of next page.
 * @param x Address of current page.
 */
//#define PFN_UP(x)       (((x) + PAGE_SIZE-1) >> PAGE_SHIFT)
/**
 * Macro to align the supplied address to the next page boundary
 * @param x Address to align.
 */
//#define PFN_ALIGN(x)    (((unsigned long)(x) + (PAGE_SIZE - 1)) & PAGE_MASK)

/**
 * Variable containing the address of the end of the kernel data
 */
//extern int _end;

/**
 * Variable to store the size of the RAM.
 */
unsigned long mem_size;

/**
 * The xtime clock lock used to lock the xtime structure while it is being updated.
 */
//extern rwlock_t xtime_lock;

/**
 * Declaration of the MIPS standard interrupt dispatch function.
 * This function is defined in int-handler.S.
 */
extern asmlinkage void lx_handle_int(void);

/**
 * Declaration of the Lexra vectored interrupt dispatch function.
 * This function is defined in int-handler.S.
 */
extern asmlinkage void lx_vec_handle_int(void);
extern asmlinkage void write_lx_intvec(unsigned int reg);
extern asmlinkage void read_lx_intvec(unsigned int *reg);
extern asmlinkage void lx_int_vec8(void);
extern asmlinkage void handle_ri(void);
extern char except_vec3_generic;	/* for reserved instruction exception */
void trihidtv_init_int(void);

/**
 * An array of lxint_t structures which store the masking information
 * for each of the 32 Linux interrupts. This data is used to enable/disable
 * either a MIPS, Lexra vectored or PCI interrupt.
 */
lxint_t lx_interrupt[NR_INTS];

/**
 * Unimplemented power control function.
 */
//extern void lx_machine_restart(char *command);
/**
 * Unimplemented power control function.
 */
//extern void lx_machine_halt(void);
/**
 * Unimplemented power control function.
 */
//extern void lx_machine_power_off(void);

//extern void wbflush_setup(void);

/*
 * Alan Liu
 * add the following declaration of the extern functions.
 */
extern void (*board_time_init) (void);
extern void trihidtv_time_init(void);

//extern void (*_machine_restart)(char *command);
//extern void (*_machine_halt)(void);
//extern void (*pm_power_off)(void);

/* 
 * Alan Liu
 * Comment the following line,and replace lx45xxx_rtc_ops
 * With lx5180_rtc_ops
 */
/* extern struct rtc_ops lx45xxx_rtc_ops; */
//extern struct rtc_ops lx5180_rtc_ops;

/**
 * External function to send a character to the UART.
 */
//extern void serial_putc(int chan, int c);

/**
 * Variable to store the limits of the initial ramdisk.
 */
#if 0
extern unsigned long initrd_start, initrd_end;
//extern void *__rd_start, *__rd_end;
extern void * __initramfs_start, * __initramfs_end;
#endif

/*
 *
 * Defined in kernel/resource.c
 * We will reset them here
 */
extern struct resource ioport_resource;
extern struct resource iomem_resource;

struct {
	struct resource ram;
	struct resource pci_mem;
	struct resource pci_io;
	struct resource flash;
	struct resource boot;
} trihidtv_resources = {
	{
		.start	= 0,
		.end	= TRIHIDTV_RAM_DEFAULT_SIZE - 1,
		.name	= "RAM",
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= TRIHIDTV_PCI_MEM_BASE,
		.end	= TRIHIDTV_PCI_MEM_BASE + TRIHIDTV_PCI_MEM_SIZE - 1,
		.name	= "PCI Mem",
		.flags	= IORESOURCE_MEM,
	},
	{
		.start	= TRIHIDTV_PCI_IO_BASE,
		.end	= TRIHIDTV_PCI_IO_BASE + TRIHIDTV_PCI_IO_SIZE - 1,
		.name	= "PCI I/O",
		.flags	= IORESOURCE_IO,
	},
};
/*
 *
 * This func will be called in arch/mips/kernel/setup.c
 * Alan Liu
 *
 * 09/18/2003
 */
#if 0
void __init
trihidtv_init_ram_resource(unsigned long memsize)
{
	trihidtv_resources.ram.end = memsize - 1;
}
#endif

extern unsigned long mips_machtype;
extern unsigned long mips_machgroup;
extern void Init_Serial(void);

void __init plat_mem_setup(void);

/**
 * This funtion initializes kernel variables with data from the boot
 * loader.
 */
void __init prom_init(void)
{
	/*
	 * Init serial port for early output...
	 */
	char *hidtvpro_def_cmd = CONFIG_CMDLINE;
	Init_Serial();

	//test_memcpy();
#if 0
#ifdef CONFIG_BLK_DEV_INITRD
        //ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0);
        initrd_start = (unsigned long) &__initramfs_start;
        initrd_end = (unsigned long) &__initramfs_end;
#endif
#endif
	if (fw_arg3 == 0) {
		strlcpy(arcs_cmdline, hidtvpro_def_cmd, COMMAND_LINE_SIZE);
	} else {
		strlcpy(arcs_cmdline, ((struct bootParams *) fw_arg3)->cmd_line, 
			COMMAND_LINE_SIZE);
	}
	/*
	 * We need process the command line parameter here??
	 * Alan Liu
	 */
	mips_machgroup = MACH_GROUP_TRIHIDTV;
	mips_machtype = MACH_TRIHIDTV;
#if 1
	/*
	 * We should get memsize from bootloader.
	 * Bootloader will get it through CONFIG table in flash...
	 * When mem=XXX[KkmM]@YYY[KkmM] is set in cmdline,then
	 * The default setting will be blown away in setup.c
	 * Alan Liu
	 */
	mem_size = TRIHIDTV_RAM_DEFAULT_SIZE;
	printk("Default %d MB SDRAM,parse mem=xxx[MmKkGg] later...\n",
	       (int) (mem_size >> 20));
	add_memory_region(TRIHIDTV_RAM_BASE, mem_size, BOOT_MEM_RAM);
#endif
};

/**
 * Variable to store the time of the last clock update.
 */
//static long last_rtc_update;

/**
 * Lexra specific interrupt setup function. This function calls the
 * routine which initializes the Lexra interrupt handling structures.
 * Following this the MIPS interrupt exception vector is installed.
 *
 * This function is called by init_IRQ() in irq.c.
 *
 * Alan Liu
 * 09/10/2003
 */
void __init
lx_irq_setup(void)
{
	int i;
	switch (mips_machgroup) {
		/*
		 * Modified by Alan Liu
		 */
		//case MACH_GROUP_LEXRA:
	case MACH_GROUP_TRIHIDTV:
		trihidtv_init_int();
		break;
	}
	/* 
	 *
	 * setupt the interrupt exception vector to dispatch 
	 * MIPS type interrupts 
	 *
	 * 0 is the ExcCode in LX5180,is also the index in array
	 * exception_handlers[] which is defined in arch/mips/kernel/traps.c and
	 * is refered in arch/mips/kernel/entry.S.
	 *
	 * lx_handle_int only deals with mips standard interrupts which the 
	 * standard entrypoint is 0x80000080.
	 *
	 * write_lx_intvec() is define in int-handlers.S(arch/mips/trihidtv/),used
	 * to write value into EINTVEC register in CP0,and lx_int_vec8 is defined in
	 * arch/mips/trihidtv/int-handler.S,its address should be 0x80100700 
	 * now.
	 *
	 * Alan Liu
	 * 09/11/2003
	 *
	 */
	/*
	 * In arch/kernel/traps.c, we set handle_int as the default handler
	 * Now we change it to lx_handle_int
	 * Alan Liu
	 * 2007/7/18
	 */
	set_except_vector(0, lx_handle_int);
	write_lx_intvec(((unsigned int) lx_int_vec8) & 0xffffffc0);
	/*
	 * Modified by Alan liu
	 * we should disable all interrupts.
	 * it will be enabled when setup_irq or request_irq is called,
	 * the controller->startup_irq() will be called.
	 * so we should use NR_IRQS,instead NR_INTS.
	 */
	for (i = 0; i < NR_IRQS; i++) {
		disable_irq(i);
	}
}

#if 0
/**
 * Stores the command line entered at the boot prompt.
 */
char arcs_cmdline[COMMAND_LINE_SIZE] = { 0, };
#endif
/*
 * This function will be called in start_kernel()--->setup_arch(),
 *
 *
 * TriHiDTV SOC machine specific setup routine. This routine sets up function
 * pointers used by the kernel intitialization code to intialize
 * various hardware specific operations like interrupts, timers and
 * io space.
 *
 * Alan Liu
 * 2003.10
 */

//extern struct ide_ops std_ide_ops;
//extern struct ide_ops *ide_ops;

unsigned int
trihidtv_pci_addr(unsigned char bus, unsigned char dev, unsigned char func)
{
	return (bus << 16) | (dev << 11) | (func << 8);
}

extern int hidtvpro_hw_misc(void);
extern void (*flush_icache_range)(unsigned long start, unsigned long end);

void __init plat_mem_setup(void)
{
	/*
	 * Misc settings about USE,IDE,and so on...
	 */
	hidtvpro_hw_misc();
#if 0
	/* 
	 * Specific ide operation routines
	 * IDE related setting here.
	 *
	 * Alan Liu
	 * 2004/2/4
	 * it seems linux 2.6 needs no IDE..(2007/7/11)
	 */
#ifdef CONFIG_BLK_DEV_IT8172
	ide_ops = &std_ide_ops;
#endif
#endif

#if 0
	/* 
	 * Alan Liu
	 * Specific RTC routines
	 * Comment the following line,and replace lx45xxx_rtc_ops
	 * With lx5180_rtc_ops
	 */
	/* rtc_ops = &lx45xxx_rtc_ops; */
	rtc_ops = &lx5180_rtc_ops;
#endif

	/*
	 * Alan Liu
	 * Add the new board_time_init and board_timer_setup assignment
	 * Commented the original assignment of board_timer_setup 
	 */
	board_time_init = trihidtv_time_init;
	
#if 0
	/*
	 * Maybe we could support this in future...
	 * Alan Liu
	 * 2007/7/11
	 */
	_machine_restart = lx_machine_restart;
	_machine_halt = lx_machine_halt;
	pm_power_off = lx_machine_power_off;
#endif
	/*
	 * IO/MEM resources.
	 * revisit this area.
	 */
	//mips_io_port_base = KSEG1;
	set_io_port_base(KSEG1);

#if 0
	ioport_resource.start = trihidtv_resources.pci_io.start;
	ioport_resource.end = trihidtv_resources.pci_io.end;
	iomem_resource.start = trihidtv_resources.pci_mem.start;
	iomem_resource.end = trihidtv_resources.pci_mem.end;
#else
	/*
	 * It seems we dont separate IO/MEM clearly...
	 * have to do this ...
	 * Alan Liu
	 */
	ioport_resource.start = 0x10000000;
	ioport_resource.end = 0x1fffffff;
	iomem_resource.start = 0x10000000;
	iomem_resource.end = 0x1fffffff;
#endif
	/* 
	 * trap MAC instructions before trap_init is called in Linux kernel,
	 * it will be override later in trap_init 
	 */
	set_except_vector(10, handle_ri);
};

#ifdef CONFIG_PCI
/*
 * PCI controller setup
 * Alan Liu
 * 2007-9-11
 */
extern struct pci_controller trihidtv_pci_controller;

static int __init trihidtv_pcibios_init(void)
{
	printk("Trihidtv pcibios init\n");
	register_pci_controller(&trihidtv_pci_controller);
}
arch_initcall(trihidtv_pcibios_init);

#endif
/**
 * Lexra specific initialization  of interrupts.
 * Support is provided for the MIPS 6 hardware and 2 software interrups,
 * the 8 Lexra vectored interrupts
 * The interrupts are prioritized through the used of the mask and irq_nr
 * tables. This allows for simple modification of the interrupt priority
 * by changing the relative location of the interrupt in these tables.
 *
 * Alan Liu
 * 09/10/2003
 * Modified the mask and irq_nr tables
 */
void __init
trihidtv_init_int(void)
{

	/*
	 * Setup interrupt structure
	 */
#ifdef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
	/* 
	 * vectored interrupts 
	 */
	lx_interrupt[AVTIMER_INT_PIN].cpu_mask = 0;
	lx_interrupt[AVTIMER_INT_PIN].LX_mask = LX_IRQ15;
	lx_mask_tbl[0] = LX_IRQ15;
	lx_irq_nr[0] = AVTIMER_INT_PIN;

	lx_interrupt[VBS_INT0_PIN].cpu_mask = 0;
	lx_interrupt[VBS_INT0_PIN].LX_mask = LX_IRQ14;
	lx_mask_tbl[1] = LX_IRQ14;
	lx_irq_nr[1] = VBS_INT0_PIN;

	lx_interrupt[AUDIO_INTN_PIN].cpu_mask = 0;
	lx_interrupt[AUDIO_INTN_PIN].LX_mask = LX_IRQ13;
	lx_mask_tbl[2] = LX_IRQ13;
	lx_irq_nr[2] = AUDIO_INTN_PIN;

	lx_interrupt[INT_BF_NOSI_PIN].cpu_mask = 0;
	lx_interrupt[INT_BF_NOSI_PIN].LX_mask = LX_IRQ12;
	lx_mask_tbl[3] = LX_IRQ12;
	lx_irq_nr[3] = INT_BF_NOSI_PIN;

	lx_interrupt[INT_BF_SI_PIN].cpu_mask = 0;
	lx_interrupt[INT_BF_SI_PIN].LX_mask = LX_IRQ11;
	lx_mask_tbl[4] = LX_IRQ11;
	lx_irq_nr[4] = INT_BF_SI_PIN;

	lx_interrupt[LXUART_INT_PIN].cpu_mask = 0;
	lx_interrupt[LXUART_INT_PIN].LX_mask = LX_IRQ10;
	lx_mask_tbl[5] = LX_IRQ10;
	lx_irq_nr[5] = LXUART_INT_PIN;

	lx_interrupt[UMACDMA_INT_PIN].cpu_mask = 0;
	lx_interrupt[UMACDMA_INT_PIN].LX_mask = LX_IRQ9;
	lx_mask_tbl[6] = LX_IRQ9;
	lx_irq_nr[6] = UMACDMA_INT_PIN;

	lx_interrupt[MI2C_1_INT_PIN].cpu_mask = 0;
	lx_interrupt[MI2C_1_INT_PIN].LX_mask = LX_IRQ8;
	lx_mask_tbl[7] = LX_IRQ8;
	lx_irq_nr[7] = MI2C_1_INT_PIN;

	/* 
	 * The MIPS standard interrupts 
	 */
	lx_interrupt[VIDEO_INTN_PIN].cpu_mask = IE_IRQ5;
	lx_interrupt[VIDEO_INTN_PIN].LX_mask = 0;
	cpu_mask_tbl[0] = IE_IRQ5;
	cpu_irq_nr[0] = VIDEO_INTN_PIN;

	lx_interrupt[DMINT_INT_PIN].cpu_mask = IE_IRQ4;
	lx_interrupt[DMINT_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[1] = IE_IRQ4;
	cpu_irq_nr[1] = DMINT_INT_PIN;

	lx_interrupt[MI2C_0_INT_PIN].cpu_mask = IE_IRQ3;
	lx_interrupt[MI2C_0_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[2] = IE_IRQ3;
	cpu_irq_nr[2] = MI2C_0_INT_PIN;

	lx_interrupt[M_S_E_INT_PIN].cpu_mask = IE_IRQ2;
	lx_interrupt[M_S_E_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[3] = IE_IRQ2;
	cpu_irq_nr[3] = M_S_E_INT_PIN;

	lx_interrupt[UNMAP_INT_PIN].cpu_mask = IE_IRQ1;
	lx_interrupt[UNMAP_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[4] = IE_IRQ1;
	cpu_irq_nr[4] = UNMAP_INT_PIN;

	lx_interrupt[M_S_S_INT_PIN].cpu_mask = IE_IRQ0;
	lx_interrupt[M_S_S_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[5] = IE_IRQ0;
	cpu_irq_nr[5] = M_S_S_INT_PIN;

	/* 
	 * IP[1]~~~IP[0] are software issued interruts 
	 */
	lx_interrupt[SW1_PIN].cpu_mask = IE_SW1;
	lx_interrupt[SW1_PIN].LX_mask = 0;
	cpu_mask_tbl[6] = IE_SW1;
	cpu_irq_nr[6] = SW1_PIN;

	lx_interrupt[SW0_PIN].cpu_mask = IE_SW0;
	lx_interrupt[SW0_PIN].LX_mask = 0;
	cpu_mask_tbl[7] = IE_SW0;
	cpu_irq_nr[7] = SW0_PIN;
#else
	/* 
	 * vectored interrupts 
	 */
	lx_interrupt[CPU_NMI_PIN].cpu_mask = 0;
	lx_interrupt[CPU_NMI_PIN].LX_mask = LX_IRQ15;
	lx_mask_tbl[0] = LX_IRQ15;
	lx_irq_nr[0] = CPU_NMI_PIN;

	lx_interrupt[VBS_INT0_PIN].cpu_mask = 0;
	lx_interrupt[VBS_INT0_PIN].LX_mask = LX_IRQ14;
	lx_mask_tbl[1] = LX_IRQ14;
	lx_irq_nr[1] = VBS_INT0_PIN;

	lx_interrupt[AUDIO_INTN_PIN].cpu_mask = 0;
	lx_interrupt[AUDIO_INTN_PIN].LX_mask = LX_IRQ13;
	lx_mask_tbl[2] = LX_IRQ13;
	lx_irq_nr[2] = AUDIO_INTN_PIN;

	lx_interrupt[INT_BF_NOSI_PIN].cpu_mask = 0;
	lx_interrupt[INT_BF_NOSI_PIN].LX_mask = LX_IRQ12;
	lx_mask_tbl[3] = LX_IRQ12;
	lx_irq_nr[3] = INT_BF_NOSI_PIN;

	lx_interrupt[INT_BF_SI_PIN].cpu_mask = 0;
	lx_interrupt[INT_BF_SI_PIN].LX_mask = LX_IRQ11;
	lx_mask_tbl[4] = LX_IRQ11;
	lx_irq_nr[4] = INT_BF_SI_PIN;

	lx_interrupt[INT0_PIN].cpu_mask = 0;
	lx_interrupt[INT0_PIN].LX_mask = LX_IRQ10;
	lx_mask_tbl[5] = LX_IRQ10;
	lx_irq_nr[5] = INT0_PIN;

	lx_interrupt[EXT_INT1_PIN].cpu_mask = 0;
	lx_interrupt[EXT_INT1_PIN].LX_mask = LX_IRQ9;
	lx_mask_tbl[6] = LX_IRQ9;
	lx_irq_nr[6] = EXT_INT1_PIN;

	lx_interrupt[EXT_INT2_PIN].cpu_mask = 0;
	lx_interrupt[EXT_INT2_PIN].LX_mask = LX_IRQ8;
	lx_mask_tbl[7] = LX_IRQ8;
	lx_irq_nr[7] = EXT_INT2_PIN;

	/* 
	 * The MIPS standard interrupts 
	 */
	lx_interrupt[VIDEO_INTN_PIN].cpu_mask = IE_IRQ5;
	lx_interrupt[VIDEO_INTN_PIN].LX_mask = 0;
	cpu_mask_tbl[0] = IE_IRQ5;
	cpu_irq_nr[0] = VIDEO_INTN_PIN;

	lx_interrupt[DMINT_INT_PIN].cpu_mask = IE_IRQ4;
	lx_interrupt[DMINT_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[1] = IE_IRQ4;
	cpu_irq_nr[1] = DMINT_INT_PIN;

	lx_interrupt[PC_HIDTV_INT_PIN].cpu_mask = IE_IRQ3;
	lx_interrupt[PC_HIDTV_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[2] = IE_IRQ3;
	cpu_irq_nr[2] = PC_HIDTV_INT_PIN;

	lx_interrupt[S_M_E_INT_PIN].cpu_mask = IE_IRQ2;
	lx_interrupt[S_M_E_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[3] = IE_IRQ2;
	cpu_irq_nr[3] = S_M_E_INT_PIN;
	/* IP[3] reserved  */
	lx_interrupt[S_M_S_INT_PIN].cpu_mask = IE_IRQ0;
	lx_interrupt[S_M_S_INT_PIN].LX_mask = 0;
	cpu_mask_tbl[5] = IE_IRQ0;
	cpu_irq_nr[5] = S_M_S_INT_PIN;

	/* 
	 * IP[1]~~~IP[0] are software issued interruts 
	 */
	lx_interrupt[SW1_PIN].cpu_mask = IE_SW1;
	lx_interrupt[SW1_PIN].LX_mask = 0;
	cpu_mask_tbl[6] = IE_SW1;
	cpu_irq_nr[6] = SW1_PIN;

	lx_interrupt[SW0_PIN].cpu_mask = IE_SW0;
	lx_interrupt[SW0_PIN].LX_mask = 0;
	cpu_mask_tbl[7] = IE_SW0;
	cpu_irq_nr[7] = SW0_PIN;
#endif
}
