/*
 * Platform device support for Au1x00 SoCs.
 *
 * Copyright 2004, Matt Porter <mporter@kernel.crashing.org>
 *
 * This file is licensed under the terms of the GNU General Public
 * License version 2.  This program is licensed "as is" without any
 * warranty of any kind, whether express or implied.
 */
#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/resource.h>

#include <asm/trihidtv/trihidtv.h>
#include <asm/trihidtv/trihidtv_addrspace.h>
#include <asm/trihidtv/trihidtv_int.h>

/* EHCI (USB high speed host controller) */
static struct resource trihidtv_usb_ehci_resources[] = {
	[0] = {
		.start		= TRIHIDTV_EHCI_BASE,
		.end		= TRIHIDTV_EHCI_BASE + TRIHIDTV_EHCI_LEN - 1,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= TRIHIDTV_USB_IRQ,
		.end		= TRIHIDTV_USB_IRQ,
		.flags		= IORESOURCE_IRQ,
	},
};

static u64 ehci_dmamask = ~(u32)0;

static struct platform_device trihidtv_usb_ehci_device = {
	.name		= "trihidtv-ehci",
	.id		= 0,
	.dev = {
		.dma_mask		= &ehci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(trihidtv_usb_ehci_resources),
	.resource	= trihidtv_usb_ehci_resources,
};


static struct platform_device *trihidtv_platform_devices[] __initdata = {
	&trihidtv_usb_ehci_device,
};

int trihidtv_platform_init(void)
{
	return platform_add_devices(trihidtv_platform_devices, ARRAY_SIZE(trihidtv_platform_devices));
}

arch_initcall(trihidtv_platform_init);
