/*
 * RTC routines for ITE8172 MC146818-compatible rtc chip.
 *
 * Trident Multimedia Technologies (Shanghai) Co.Ltd, 2003
 * Modified this file for trident HiDTV chipset.
 * Alan Liu
 * 08/15/2003
 *
 *  ----------------------------------------------------------------------
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *  -----------------------------------------------------------------------
 *
 */
#include </mc146818rtc.h>		/* not used in this file */
#include <asm/addrspace.h>		/* macro KSEG1ADDR */
#include <asm/trihidtv/trihidtv.h>

#ifdef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
/*
 * Since NO RTC in AV MIPS, so we shoud emulate one RTC.
 * How we implement this feature?
 * Now Just keep it blank...
 * Alan Liu
 */
unsigned char lx5180_rtc_read_data(unsigned long addr)
{
	return 0;
}
void lx5180_rtc_write_data(unsigned char data, unsigned long addr)
{
	return;
}
static int lx5180_rtc_bcd_mode(void)
{
	return 0;
}

#else

#define LX5180_RTC_ADR_REG  0x14014800
#define LX5180_RTC_DAT_REG  (LX5180_RTC_ADR_REG + 1)

static volatile char *rtc_addr_reg = KSEG1ADDR((char *)LX5180_RTC_ADR_REG);
static volatile char *rtc_data_reg = KSEG1ADDR((char *)LX5180_RTC_DAT_REG);

unsigned char lx5180_rtc_read_data(unsigned long addr)
{
	unsigned char retval;
	
	WriteRegByte(rtc_addr_reg,addr);
	retval = ReadRegByte(rtc_data_reg);
	return retval;
}

void lx5180_rtc_write_data(unsigned char data, unsigned long addr)
{
	WriteRegByte(rtc_addr_reg,addr);
	WriteRegByte(rtc_data_reg,data);
}

/*
 * use binary mode
 */
static int lx5180_rtc_bcd_mode(void)
{
	return 0;
}
#endif	/* CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS */
/*
 * lx5180_rtc_ops is refered in arch/mips/lx/lx.c,lexra_setup()
 */
struct rtc_ops lx5180_rtc_ops = {
	&lx5180_rtc_read_data,
	&lx5180_rtc_write_data,
	&lx5180_rtc_bcd_mode
};
