/*
 *  drivers/mtd/nand/hidtv_nand.c
 *
 * $Id: hidtv_nand.c,v 1.1 2008/08/04 11:50:31 samsheng Exp $
 *
 *  Overview:
 *   This is a device driver for the NAND flash device found on the
 *   HiDTV board
 *  Alan Liu(alanliu@trident.com.cn) 10-06-2004 
 *		initial draft,need more information to enable NAND on HiDTV
 */

#include <linux/slab.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/nand.h>
#include <linux/mtd/partitions.h>
#include <asm/io.h>
//Alan Liu
#include <asm/trihidtv/trihidtv.h>
#include <asm/trihidtv/trihidtv_gpio.h>

/*
 * MTD structure for HiDTV board
 */
static struct mtd_info *hidtv_nand_mtd = NULL;

#define HIDTV_NAND_PHY_ADDR	CONFIG_MTD_NAND_PHYSMAP_START	/* Start of HIDTV NAND IO address space */
#define HIDTV_NAND_LEN		CONFIG_MTD_NAND_PHYSMAP_LEN	/* Size of HIDTV NAND */

/*
 * Module stuff
 */

/*
 * Define partitions for flash device
 */
#ifdef CONFIG_MTD_PARTITIONS

struct mtd_partition nand_physical_partition_info[] = {
	{
		name:"Whole physical Flash(16M) on NAND",
		offset:0,
		size:16*1024*1024
	}
};
struct mtd_partition nand_partition_info[] = {
	{
		name:"Boot(bootloader+kernel+etc.)(6M) On NAND",
		offset:0,
		size:6*1024*1024
	},
	{
		name:"JFFS2 file system(8M) On NAND",
		offset:6*1024*1024,
		size:8*1024*1024
	},
	{
		name:"Initial Ramdisk image(2M) On NAND",
		offset:14*1024*1024,
		size:2*1024*1024
	}
};

#define NUM_PARTITIONS (sizeof(nand_partition_info)/sizeof(struct mtd_partition))

#endif
///////////////////////////////////////////////////////////////////
//Alan Liu
///////////////////////////////////////////////////////////////////
#if 0
void
EnableALE(int on)
{
	unsigned short val;
	val=ReadRegHWord(0x14013002);
	if(on){
		WriteRegHWord(0x14013002,val|0x20);
	}
	else{
		WriteRegHWord(0x14013002,val&~0x20);
	}
}

void
EnableCLE(int on)
{
	unsigned short val;
	val=ReadRegHWord(0x14013002);
	if(on){
		WriteRegHWord(0x14013002,val|0x02);
	}
	else{
		WriteRegHWord(0x14013002,val&~0x02);
	}
}

void
EnableNCE(int on)
{
	unsigned short val;
	val=ReadRegHWord(0x14013002);
	if(on){
		WriteRegHWord(0x14013002,val|0x1);
	}
	else{
		WriteRegHWord(0x14013002,val&~0x1);
	}
}

static
void 
nand_set_gpio_mode()
{
	unsigned short val;
	//SET GPIO 0 to output mode...
	val = ReadRegHWord(0x14013000);
	val &= ~0x03;
	val |= 0x01;
	WriteRegHWord(0x14013000,val);
}
#endif

static void
enable_nce(int enabled)
{
	unsigned int val;
	
	val = ReadRegWord(0x1b002000);
	if(!enabled){
		WriteRegHWord(0x1b002000, val | 0x80);
	}
	else{
		WriteRegHWord(0x1b002000, val & ~0x80);
	}
}

/*
 *	hidtv_prepare_for_nand
 *	Set registers in Hidtv to support Nand
 */
static int
hidtv_prepare_for_nand(void)
{
	//set 8 bit wide flash
	WriteRegWord(0x1b002004, 0x00000021); /*flash 8 bits wide,and access timing is 40ns,refer to SPG*/
	//WriteRegWord(0x1b002004, 0x00000067);   /*flash 8 bits wide,and access timing is 120ns,refer to SPG*/
	//Disable NOR flash at first
        WriteRegWord(0x1b004000, 0x00000e60);
        WriteRegWord(0x1b004004, 0x003fffff);

	//Timing issue...
	//Ask Ruoyu/ChenXiang about the detail...
	//Need put these into EEPROM
	//Alan Liu
	//From Chen Xiang:
	//1.) In your EEPROM, at the starting place,
	//please add: Write dc5a08 to b5022fa4(word access);
	//Write f18 to bb00001a(halfword access); Write 0x60 to 150260c4(Byte access).
#if 0
	WriteRegWord(0x15022fa4,0xdc5a08);
	WriteRegHWord(0x1b00001a,0xf18);
	WriteRegByte(0x150260c4,0x60);
#endif

#if 0
	nand_set_gpio_mode();
	//set GPIO 0 default status
	EnableNCE(1);
#endif

	return 0;
}

///////////////////////////////////////////////////////////////////
/* 
 *	hardware specific access to control-lines
 */
#if 0
static void hidtv_nand_hwcontrol(struct mtd_info *mtd, int cmd) 
{
	struct nand_chip* this = (struct nand_chip *) (mtd->priv);
	
	switch(cmd) {
		
	case NAND_CTL_SETCLE: 
		//EnableCLE(1);
		this->IO_ADDR_R |= (1 << 1);
		this->IO_ADDR_W |= (1 << 1);
		break;
	case NAND_CTL_CLRCLE: 
		//EnableCLE(0);
		this->IO_ADDR_R &= ~(1 << 1);
		this->IO_ADDR_W &= ~(1 << 1);
		break;
		
	case NAND_CTL_SETALE:
		//EnableALE(1);
		this->IO_ADDR_R |= (1 << 0);
		this->IO_ADDR_W |= (1 << 0);
		break;
	case NAND_CTL_CLRALE:
		//EnableALE(0);
		this->IO_ADDR_R &= ~(1 << 0);
		this->IO_ADDR_W &= ~(1 << 0);
		break;
		
	case NAND_CTL_SETNCE:
		EnableNCE(0);
		break;

	case NAND_CTL_CLRNCE:
		EnableNCE(1);
		break;
	}
}
#else
static void hidtv_nand_hwcontrol(struct mtd_info *mtd, int cmd, unsigned int ctrl)
{
	struct nand_chip *nand_chip = (struct nand_chip *) (mtd->priv);
	static unsigned long IO_ADDR_W = 0;
	
	switch (ctrl) {
	case NAND_CTRL_CHANGE:
		enable_nce(0);
		break;
	
	default:
		enable_nce(1);
		if(ctrl & NAND_CTRL_CHANGE)
		{
			if(ctrl == NAND_NCE)
				IO_ADDR_W = ((unsigned long)nand_chip->IO_ADDR_W);
			else if(ctrl & NAND_ALE)
				IO_ADDR_W = ((unsigned long)nand_chip->IO_ADDR_W) + (1 << 0);
			else if(ctrl & NAND_CLE)
				IO_ADDR_W = ((unsigned long)nand_chip->IO_ADDR_W) + (1 << 1);
			else
				IO_ADDR_W = ((unsigned long)nand_chip->IO_ADDR_W);
		}
		
		break;
	}
	
	if (cmd != NAND_CMD_NONE)
		writeb(cmd, (void *)IO_ADDR_W);
}


int hidtv_nand_ready(struct mtd_info *mtd)
{
	return (ReadRegWord(0x1b002000) & 0x40) != 0;
}

#endif

/*
 *	read device ready pin
 *	I think we should feedback R/B bit to a GPIO,
 *	Then we could check the nand device ready status.
 *	Alan Liu
 *	2004-9-22	
 */
#if 0
static int hidtv_nand_device_ready(struct mtd_info *mtd)
{
	return ???;
}
#endif

#ifdef CONFIG_MTD_NAND_CRAMFS_SUPPORT
#ifdef CONFIG_CRAMFS
	extern unsigned int nand_tell_cramfs;
	extern unsigned int cramfs_set_mapping_table();
#endif	
#endif	
/*
 * Main initialization routine
 */
static int __init hidtv_nand_init (void)
{
	struct nand_chip *this;
#ifndef CONFIG_MTD_PARTITIONS
	const char *part_type = 0;
	int mtd_parts_nb = 0;
	struct mtd_partition *mtd_parts = 0;
#endif
	void* nandaddr;
#if 0	
	if (!machine_is_hidtv())
		return -ENODEV;
#endif		
	nandaddr = (void *)ioremap(HIDTV_NAND_PHY_ADDR, HIDTV_NAND_LEN);
	if (!nandaddr)
	{
		printk("Failed to ioremap in nand flash.\n");
		return -ENOMEM;
	};
	
	/* Allocate memory for MTD device structure and private data */
	hidtv_nand_mtd = kmalloc(sizeof(struct mtd_info) + 
			     sizeof(struct nand_chip),
			     GFP_KERNEL);
	if (!hidtv_nand_mtd) {
		printk("Unable to allocate HIDTV NAND MTD device structure.\n");
		return -ENOMEM;
	}
		
	/* Get pointer to private data */
	this = (struct nand_chip *) (&hidtv_nand_mtd[1]);
	//debug here.
	//printk("%s:hidtv_nand_mtd is %x,nand_chip is %x\n",__FUNCTION__,hidtv_nand_mtd,this);
	
	/* Initialize structures */
	memset((char *) hidtv_nand_mtd, 0, sizeof(struct mtd_info));
	memset((char *) this, 0, sizeof(struct nand_chip));
	
	/* Link the private data with the MTD structure */
	hidtv_nand_mtd->priv = this;

	/*
	 * nand related setting before accessing it physically
	 * Alan Liu
	 */
	hidtv_prepare_for_nand();
	
	/* insert callbacks */
	this->IO_ADDR_R = nandaddr;
	this->IO_ADDR_W = nandaddr;
	this->cmd_ctrl = hidtv_nand_hwcontrol;
	//
	//Set this function in future,
	//Maybe assign the R/B pin to a GPIO.
	//Alan Liu
	//
	//this->dev_ready = NULL;	/* unknown whether that was correct or not so we will just do it like this */
	this->dev_ready = hidtv_nand_ready;
	/* 15 us command delay time */
	this->chip_delay = 50;
	this->ecc.mode = NAND_ECC_SOFT;
	//this->ecc.mode = NAND_ECC_NONE;
	this->options = NAND_NO_AUTOINCR;

	//
	//Set this option to force FLASH based bbt
	//Then we could speed up the bootup,
	//Everytime a block is marked bad,
	//it should rewrite the bad table.
	//Alan Liu
	this->options |= NAND_USE_FLASH_BBT;
	
	/* Scan to find existence of the device */
	if (nand_scan (hidtv_nand_mtd, 1)) {
		printk(KERN_NOTICE "No NAND device - returning -ENXIO\n");
		kfree (hidtv_nand_mtd);
		return -ENXIO;
	}
	
#if 0
#ifdef CONFIG_MTD_CMDLINE_PARTS
	mtd_parts_nb = parse_cmdline_partitions(hidtv_nand_mtd, &mtd_parts, 
						"h1910-nand");
	if (mtd_parts_nb > 0)
	  part_type = "command line";
	else
	  mtd_parts_nb = 0;
#endif
#endif

#ifdef CONFIG_MTD_PARTITIONS

	{
		struct mtd_partition	*parts = NULL;
		int			nr_parts = 0;

#ifdef CONFIG_MTD_CMDLINE_PARTS
		static const char *part_probes[] = { "cmdlinepart", NULL, };

		nr_parts = parse_mtd_partitions(hidtv_nand_mtd,
				part_probes, &parts, 0);
#endif

		if (nr_parts > 0) {
			return add_mtd_partitions(hidtv_nand_mtd, parts, nr_parts);
		}
	}
	
#else

	part_type = "static";
	printk(KERN_NOTICE "Using %s partition definition\n", part_type);

	/*
	 * The whole nand flash...
	 */
	/* Register the partition */
	add_mtd_partitions(hidtv_nand_mtd, nand_physical_partition_info, 1);

	/*
	 *
	 */
	mtd_parts = nand_partition_info;
	mtd_parts_nb = NUM_PARTITIONS;
	/* Register the partitions */
	add_mtd_partitions(hidtv_nand_mtd, mtd_parts, mtd_parts_nb);
#endif

#ifdef CONFIG_MTD_NAND_CRAMFS_SUPPORT
#ifdef CONFIG_CRAMFS
	/*
	 * Always put this to last!
	 */
	printk("%s:Tell CRAMFS that nand is ready.\n",__FUNCTION__);
	nand_tell_cramfs = 1;
	cramfs_set_mapping_table();
#endif	
#endif
	/* Return happy */
	return 0;
}

module_init(hidtv_nand_init);

/*
 * Clean up routine
 */
#ifdef MODULE

static void __exit
hidtv_nand_cleanup(void)
{
	struct nand_chip *this = (struct nand_chip *) &hidtv_nand_mtd[1];

	/* Unregister the device */
	del_mtd_device(hidtv_nand_mtd);

	/* Free internal data buffer */
	kfree(this->data_buf);

	/* Free the MTD device structure */
	kfree(hidtv_nand_mtd);
#ifdef CONFIG_MTD_NAND_CRAMFS_SUPPORT
#ifdef CONFIG_CRAMFS
	printk("%s:Tell CRAMFS that nand is over.\n",__FUNCTION__);
	nand_tell_cramfs = 0;
#endif	
#endif	
}
module_exit(hidtv_nand_cleanup);

#endif

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Alan Liu(Liu Hongming):alanliu@trident.com.cn");
MODULE_DESCRIPTION("driver for NAND on HiDTV board");
