/***************************************************************************
*   File:    trihidtv.h  -   Common definition(Endian mode) for 
*   Trident HiDTV PRO CX
* 
*   The content of this file or document is CONFIDENTIAL and PROPRIETARY
*   to Trident Microsystems, Inc.  It is subject to the terms of a
*   License Agreement between Licensee and Trident Microsystems, Inc.
*   restricting among other things, the use, reproduction, distribution
*   and transfer.  Each of the embodiments, including this information 
*   and any derivative work shall retain this copyright notice.
* 
*   Copyright (c) 2006 Trident Microsystems, Inc. 
*   All rights reserved.
****************************************************************************

DESCRIPTION:  

	Since new endian(pure big endian) is supported in HiDTV PRO CX,
	this file should be modified,and the following macros need be
	defined in advance:

	CONFIG_ENDIAN_LEGACY:		HiDTV and HiDTV-PRO compatible
	CONFIG_ENDIAN_BIGENDIAN: 	HiDTV-PRO-CX new endian mode.

Modification history
--------------------
2003/07/29  -  MGN,Teressa  add macro TOVIRTADRS to handle Apps' requests.
2003/07/25  -  MGN add trihidtv registers access routines supporting.
2003/07/24  -  MGN initial created: add endian handle macros.
2004/03/09  -  Alan Liu: modified this file,new & fast macro.
2006/08/10  -  Alan Liu: Redesign this file to adapt HiDTV PRO CX and its
			 descendant chips...
****************************************************************************/

#ifndef _TRIHIDTV_H

#define _TRIHIDTV_H

#define __INLINE_REG_ACCESS     /* use inline version of register access */

/*
** basic type defintions 
*/
#define BYTE    unsigned char
#define HWORD   unsigned short
#define WORD    unsigned int
#define PBYTE   BYTE*
#define PHWORD  HWORD*
#define PWORD   WORD*

#define ADDRS   unsigned int
#define PADDRS  ADDRS*

#define Uint8	unsigned char
#define Sint8	char
#define Uint16	unsigned short
#define Sint16	short
#define Uint32	unsigned int
#define Sint32	int
#define Uint64	unsigned long long
#define Sint64	long long

/*
** address convertion 
*/
#define VIRTTOPHYS(adr)     ((adr)&0x1fffffff)
#define PHYSTOKSEG0(adr)    ((adr)|0x80000000)
#define PHYSTOKSEG1(adr)    ((adr)|0xA0000000)

/*
** -MGN- 20030729, add here for that Teressa propse to handle Apps requests 
**          such as access the registers with a mmapped address. 
*/
//#define IN_BOOTLOADER   1
#ifndef __KERNEL__
#define __KERNEL__  1
#endif

#if defined(__KERNEL__) || defined(IN_BOOTLOADER)
#define TOVIRTADRS(adr)     PHYSTOKSEG1(adr) 
#else
/** in this case, user should ensure the passed address is virtual address **/ 
#define TOVIRTADRS(adr)     (adr) 
#endif

/*
** define register table type 
*/
#define REGRECORD   0       /* record type mode */
#define REGBITMAP   1       /* a compression type as a bitmap format */
#define REGSVGTBL   2       /* svp register table generated by resource builder*/

/*
** register access length 
*/
#define REGLEN1     1       /* BYTE access type  */
#define REGLEN2     2       /* HWORD access type */
#define REGLEN4     4       /* WORD access type  */

#ifndef __KERNEL__

#define _swap2(x)   ((((x)>>8)&0xff)||(((x)&0xff)<<8))
#define _swap4(x)   \
    ((((x)&0xff)<<24)||(((x)&0xff00)<<8)||(((x)>>8)&0xff00)||(((x)>>24)&0xff))

#define le16_to_cpu(x) \
	((Uint16)((((Uint16)(x) & 0x00FFU) << 8) | \
		  (((Uint16)(x) & 0xFF00U) >> 8)))
 
#define le32_to_cpu(x) \
	((Uint32)((((Uint32)(x) & 0x000000FFU) << 24) | \
		  (((Uint32)(x) & 0x0000FF00U) <<  8) | \
		  (((Uint32)(x) & 0x00FF0000U) >>  8) | \
		  (((Uint32)(x) & 0xFF000000U) >> 24)))

#define le64_to_cpu(x) \
	((Uint64)((((Uint64)(x) & 0x00000000000000FFULL) << 56) | \
		  (((Uint64)(x) & 0x000000000000FF00ULL) << 40) | \
		  (((Uint64)(x) & 0x0000000000FF0000ULL) << 24) | \
		  (((Uint64)(x) & 0x00000000FF000000ULL) <<  8) | \
		  (((Uint64)(x) & 0x000000FF00000000ULL) >>  8) | \
		  (((Uint64)(x) & 0x0000FF0000000000ULL) >> 24) | \
		  (((Uint64)(x) & 0x00FF000000000000ULL) >> 40) | \
		  (((Uint64)(x) & 0xFF00000000000000ULL) >> 56)))		

#define cpu_to_le16(x) (le16_to_cpu(x))
#define cpu_to_le32(x) (le32_to_cpu(x))
#define cpu_to_le64(x) (le64_to_cpu(x))

#define __constant_cpu_to_le16(x) (le16_to_cpu(x))
#define __constant_cpu_to_le32(x) (le32_to_cpu(x))
#define __constant_cpu_to_le64(x) (le64_to_cpu(x))

#endif

/*
** define common read/write micros for TriHiDTV chip according to 
** different endian configurations.
*/
#if defined(CONFIG_ENDIAN_BIG) || defined(CONFIG_CPU_BIG_ENDIAN)

#define TDRDBYTE(adr)       (*(volatile BYTE*)(adr))
#define TDRDHWORD(adr)      (*(volatile HWORD*)(adr))
#define TDRDWORD(adr)       (*(volatile WORD*)(adr))

#define TDWTBYTE(adr,v)     *(volatile BYTE*)(adr)  = v 
#define TDWTHWORD(adr,v)    *(volatile HWORD*)(adr) = v
#define TDWTWORD(adr,v)     *(volatile WORD*)(adr)  = v

#endif

#ifdef CONFIG_ENDIAN_LEGACY

#define TDRDBYTE(adr)       (*(volatile BYTE*)((adr)^0x3))
#define TDRDHWORD(adr)      (*(volatile HWORD*)((adr)^0x2))
#define TDRDWORD(adr)       (*(volatile WORD*)(adr))

#define TDWTBYTE(adr,v)     *(volatile BYTE*)((adr)^0x3)=v
#define TDWTHWORD(adr,v)    *(volatile HWORD*)((adr)^0x2)=v
#define TDWTWORD(adr,v)     *(volatile WORD*)(adr) = v


#endif

/*
**  Register access macros force to use virtual address 
*/
#define TDVIRTRDBYTE(adr)       TDRDBYTE(TOVIRTADRS(adr))
#define TDVIRTRDHWORD(adr)      TDRDHWORD(TOVIRTADRS(adr))
#define TDVIRTRDWORD(adr)       TDRDWORD(TOVIRTADRS(adr))

#define TDVIRTWTBYTE(adr,v)     TDWTBYTE(TOVIRTADRS(adr),v)
#define TDVIRTWTHWORD(adr,v)    TDWTHWORD(TOVIRTADRS(adr),v)
#define TDVIRTWTWORD(adr,v)     TDWTWORD(TOVIRTADRS(adr),v)

/*
**  C routines prototype declaration and type definitions
*/
typedef struct {
    ADDRS   addr;
    WORD    val;
} TRIREGRECORD, *PTRIREGRECORD;

/*
**  register access routins
*/
#ifndef __INLINE_REG_ACCESS
extern BYTE  ReadRegByte(ADDRS addr);
extern HWORD ReadRegHWord(ADDRS addr);
extern WORD  ReadRegWord(ADDRS addr);

extern void  WriteRegByte(ADDRS addr, BYTE val);
extern void  WriteRegHWord(ADDRS addr, HWORD val);
extern void  WriteRegWord(ADDRS addr, WORD val);

extern BYTE ReadMemByte(ADDRS addr);
extern BYTE ReadMemHWord(ADDRS addr);
extern BYTE ReadMemWord(ADDRS addr);

extern void WriteMemByte(ADDRS addr, BYTE val);
extern void WriteMemHWord(ADDRS addr, BYTE val);
extern void WriteMemWord(ADDRS addr, BYTE val);

#else

static inline  BYTE ReadRegByte(ADDRS addr)
        {return TDRDBYTE(TOVIRTADRS(addr));}

static inline  HWORD ReadRegHWord(ADDRS addr)
        {return TDRDHWORD(TOVIRTADRS(addr));}

static inline  WORD  ReadRegWord(ADDRS addr)
        {return TDRDWORD(TOVIRTADRS(addr));}

static inline  void  WriteRegByte(ADDRS addr, BYTE val)
        {TDWTBYTE(TOVIRTADRS(addr),val);}
            
static inline  void  WriteRegHWord(ADDRS addr, HWORD val)
        {TDWTHWORD(TOVIRTADRS(addr),val);}

static inline  void  WriteRegWord(ADDRS addr, WORD val)
        {TDWTWORD(TOVIRTADRS(addr),val);}

/*
 * Alan Liu
 * Add these macros to access memory...
 */
static inline  BYTE ReadMemByte(ADDRS addr)
        {return TDRDBYTE(addr);}

static inline  HWORD ReadMemHWord(ADDRS addr)
        {return TDRDHWORD(addr);}

static inline  WORD  ReadMemWord(ADDRS addr)
        {return TDRDWORD(addr);}

static inline  void  WriteMemByte(ADDRS addr, BYTE val)
        {TDWTBYTE(addr,val);}
            
static inline  void  WriteMemHWord(ADDRS addr, HWORD val)
        {TDWTHWORD(addr,val);}

static inline  void  WriteMemWord(ADDRS addr, WORD val)
        {TDWTWORD(addr,val);}

#ifdef CONFIG_ENDIAN_LEGACY

#define ReadRegByte_S	ReadRegByte
#define ReadRegHWord_S	ReadRegHWord
#define ReadRegWord_S	ReadRegWord

#define WriteRegByte_S WriteRegByte
#define WriteRegHWord_S WriteRegHWord
#define WriteRegWord_S WriteRegWord

#else

#define swap_hw(x) \
        ({ unsigned short x_ = (unsigned short)x; \
         (unsigned short)( \
                ((x_ & 0x00FFU) << 8) | ((x_ & 0xFF00U) >> 8) ); \
        })

#define swap_w(x) \
        ({ unsigned long x_ = (unsigned long)x; \
         (unsigned long)( \
                ((x_ & 0x000000FFUL) << 24) | \
                ((x_ & 0x0000FF00UL) <<  8) | \
                ((x_ & 0x00FF0000UL) >>  8) | \
                ((x_ & 0xFF000000UL) >> 24) ); \
        })

static inline  BYTE ReadRegByte_S(ADDRS addr)
		{
			BYTE b = (BYTE)TDRDBYTE(TOVIRTADRS(addr));
			/* tell compiler we need make sure to return a __BYTE__ */
			__asm__ __volatile__(".set\tnoreorder\n\t.set\treorder"::: "memory");
			return b;
		}

static inline  HWORD ReadRegHWord_S(ADDRS addr)
		{
			HWORD hw = (HWORD)swap_hw(TDRDHWORD(TOVIRTADRS(addr)));
			/* tell compiler we need make sure to return a __HWORD__ */
			__asm__ __volatile__(".set\tnoreorder\n\t.set\treorder"::: "memory");
			return hw;
		}

static inline  WORD  ReadRegWord_S(ADDRS addr)
        {return (WORD)swap_w(TDRDWORD(TOVIRTADRS(addr)));}

static inline  void  WriteRegByte_S(ADDRS addr, BYTE val)
        {TDWTBYTE(TOVIRTADRS(addr),val);}
            
static inline  void  WriteRegHWord_S(ADDRS addr, HWORD val)
        {TDWTHWORD(TOVIRTADRS(addr),swap_hw(val));}

static inline  void  WriteRegWord_S(ADDRS addr, WORD val)
        {TDWTWORD(TOVIRTADRS(addr),swap_w(val));}

#endif

#endif /*__INLINE_REG_ACCESS*/

/* return 0, on success, -l failed, with errorno set. */
extern int WriteRegTable(void* table, int tbltype, int dtype);

#endif /*_TRIHIDTV_H*/
