/*
 *
 * BRIEF MODULE DESCRIPTION
 *	Trident HiDTV SOC Interrupt Numbering
 *
 * Copyright 2003 TMT Shanghai
 * Alan Liu	alanliu@trident.com.cn
 *
 */

#ifndef _MIPS_TRIHIDTVINT_H
#define _MIPS_TRIHIDTVINT_H

#ifdef CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS
/*
 * Here's the "strategy":
 * We number 0 - 15 as the mapping between cpu irq pins and irqs
 */
/* 
 * first CPU IRQ number 
 */
#define TRIHIDTV_CPU_IRQ_BASE		0	

#define TRIHIDTV_CPU_SW0_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 0)
#define TRIHIDTV_CPU_SW1_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 1)
#define TRIHIDTV_CPU_M_S_S_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 2)
#define TRIHIDTV_CPU_UNMAP_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 3)
#define TRIHIDTV_CPU_M_S_E_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 4)
#define TRIHIDTV_CPU_MI2C_0_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 5)
#define TRIHIDTV_CPU_DMINT_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 6)
#define TRIHIDTV_CPU_VIDEO_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 7)

#define TRIHIDTV_CPU_MI2C_1_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 8)
#define TRIHIDTV_CPU_UMACDMA_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 9)
#define TRIHIDTV_CPU_LXUART_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 10)
#define TRIHIDTV_CPU_BF_SI_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 11)
#define TRIHIDTV_CPU_BF_NOSI_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 12)
#define TRIHIDTV_CPU_AUDIO_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 13)
#define TRIHIDTV_CPU_VBS_INT0_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 14)
#define TRIHIDTV_CPU_AVTIMER_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 15)

#define TRIHIDTV_LAST_IRQ		(TRIHIDTV_CPU_AVTIMER_IRQ)

//as the timer(HZ) interrupt IRQ
#define TRIHIDTV_TIMER_IRQ		(TRIHIDTV_CPU_AVTIMER_IRQ)

#define IS_CPU_IRQ(x)			(( x >= 0 && x <= 7 )? 1 : 0)
#define IS_VECTOR_IRQ(x)		(( x >= 8 && x <= 15 )? 1 : 0)

#else

/*
 * Here's the "strategy":
 * We number 0 - 15 as the mapping between cpu irq pins and irqs,thereinto,
 * 10 is for INT0 and 15 for NMI,they are dispatched deeper.
 * the LPC irqs from 16 to 31,
 * the local bus irqs from 32 to 47,
 * the pci dev register interrupts from 48 to 63,
 * and the non-maskable ints from 64 to 69.
 */

#define TRIHIDTV_CPU_IRQ_BASE	0	/* first CPU int number */
/*  0 reserved */
#define TRIHIDTV_CPU_SW0_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 0)
#define TRIHIDTV_CPU_SW1_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 1)
#define TRIHIDTV_CPU_S_M_S_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 2)
/* 3 reserved */
#define TRIHIDTV_CPU_S_M_E_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 4)
#define TRIHIDTV_CPU_PC_HIDTV_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 5)
#define TRIHIDTV_CPU_DMINT_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 6)
#define TRIHIDTV_CPU_VIDEO_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 7)
/* 8 reserved */
#define TRIHIDTV_CPU_EXTINT1_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 9)
#define TRIHIDTV_CPU_INT0_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 10)	/* see below */
#define TRIHIDTV_CPU_BF_SI_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 11)
#define TRIHIDTV_CPU_BF_NOSI_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 12)
#define TRIHIDTV_CPU_AUDIO_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 13)
#define TRIHIDTV_CPU_VBS_INT0_IRQ	(TRIHIDTV_CPU_IRQ_BASE + 14)
#define TRIHIDTV_CPU_NMI_IRQ		(TRIHIDTV_CPU_IRQ_BASE + 15)	/* see below */

#define TRIHIDTV_LPC_IRQ_BASE  16	/* first LPC int number */
/*  16 - 31 reserved */

#define TRIHIDTV_LB_IRQ_BASE  32	/* first local bus int number */
#define TRIHIDTV_ADC0_IRQ         (TRIHIDTV_LB_IRQ_BASE + 0)	/* ADC0 */
#define TRIHIDTV_ADC1_IRQ         (TRIHIDTV_LB_IRQ_BASE + 1)	/* ADC1 */
#define TRIHIDTV_USB_IRQ          (TRIHIDTV_LB_IRQ_BASE + 2)	/* USB */
#define TRIHIDTV_UART1_IRQ        (TRIHIDTV_LB_IRQ_BASE + 3)	/* UART1 */
#define TRIHIDTV_GPIO_IRQ         (TRIHIDTV_LB_IRQ_BASE + 4)
#define TRIHIDTV_CIR0_IRQ         (TRIHIDTV_LB_IRQ_BASE + 5)
#define TRIHIDTV_SCR1_IRQ         (TRIHIDTV_LB_IRQ_BASE + 6)
#define TRIHIDTV_UART2_IRQ         (TRIHIDTV_LB_IRQ_BASE + 6)	/* UART2 */
#define TRIHIDTV_UART_IRQ         (TRIHIDTV_LB_IRQ_BASE + 7)
#define TRIHIDTV_SCR0_IRQ         (TRIHIDTV_LB_IRQ_BASE + 8)
#define TRIHIDTV_MI2C0_IRQ        (TRIHIDTV_LB_IRQ_BASE + 9)
#define TRIHIDTV_RTC_IRQ          (TRIHIDTV_LB_IRQ_BASE + 10)
#define TRIHIDTV_MI2C1_IRQ        (TRIHIDTV_LB_IRQ_BASE + 11)
#define TRIHIDTV_CA_IRQ		  (TRIHIDTV_LB_IRQ_BASE + 12)
/* 13 - 14 reserved */
#define TRIHIDTV_PWRON_IRQ	  (TRIHIDTV_LB_IRQ_BASE + 15)

#define TRIHIDTV_PCI_DEV_IRQ_BASE  48	/* first pci dev irq */
/* 0 reserved */
#define TRIHIDTV_MC68K_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 1)
#define TRIHIDTV_IDE_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 2)
/* 3 reserved */
#define TRIHIDTV_H2P_MASTER_ABT_IRQ	(TRIHIDTV_PCI_DEV_IRQ_BASE + 4)
#define TRIHIDTV_H2P_TARGET_ABT_IRQ	(TRIHIDTV_PCI_DEV_IRQ_BASE + 5)
#define TRIHIDTV_PCI_INTA_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 6)
#define TRIHIDTV_PCI_INTB_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 7)
#define TRIHIDTV_PCI_INTC_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 8)
#define TRIHIDTV_PCI_INTD_IRQ		(TRIHIDTV_PCI_DEV_IRQ_BASE + 9)
/* 10 - 15 reserved */

#define TRIHIDTV_NMI_IRQ_BASE      64
/* 0 reserved */
#define TRIHIDTV_PCISERR_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 1)
#define TRIHIDTV_RTCSERR_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 2)
#define TRIHIDTV_CPUIFNTD_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 3)
/* 4 reserved */
#define TRIHIDTV_POWEROFF_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 5)
/* 6 reserved */
#define TRIHIDTV_RTCALARM1_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 7)
/* 8 - 11 reserved */
#define TRIHIDTV_WATCHDOG1_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 12)
#define TRIHIDTV_WATCHDOG2_NMI_IRQ	(TRIHIDTV_NMI_IRQ_BASE + 13)
#define TRIHIDTV_TIMER1_NMI_IRQ		(TRIHIDTV_NMI_IRQ_BASE + 14)
#define TRIHIDTV_TIMER2_NMI_IRQ		(TRIHIDTV_NMI_IRQ_BASE + 15)

#define TRIHIDTV_LAST_IRQ		(TRIHIDTV_TIMER2_NMI_IRQ)

//as the timer(HZ) interrupt IRQ
#define TRIHIDTV_TIMER_IRQ		(TRIHIDTV_TIMER2_NMI_IRQ)

//Macros for INTC_TYPER register
#define INTC_TYPER_NMI	0x8
#define INTC_TYPER_PCI	0x4
#define INTC_TYPER_LPC	0x2		//not used in trihidtv soc!
#define INTC_TYPER_LBD	0x1

//
#define IS_NMI_IRQ(x)	(( x >= TRIHIDTV_NMI_IRQ_BASE && x <= (TRIHIDTV_NMI_IRQ_BASE +15))? 1 : 0)
#define IS_INT0_IRQ(x)	(( x >= TRIHIDTV_LPC_IRQ_BASE && x <= (TRIHIDTV_NMI_IRQ_BASE - 1))? 1 : 0)
#define IS_CPU_IRQ(x)	(( x >= 0 && x <= 7 )? 1 : 0)
#define IS_VECTOR_IRQ(x)	(( x >= 8 && x <= 15 )? 1 : 0)
//#define IRQS_TO_CPU_INT(x) ( IS_NMI_IRQ(x) ? CPU_NMI_PIN : ( IS_INT0_IRQ(x) ? INT0_PIN : x ) )

/* 
 * ITE Interrupt Controller Registers
 */
struct ite_intc_regs {
	volatile unsigned short lb_req;		/* offset 0 */
	volatile unsigned short lb_mask;
	volatile unsigned short lb_trigger;
	volatile unsigned short lb_level;
	unsigned char pad0[8];

	volatile unsigned short lpc_req;	/* offset 0x10 */
	volatile unsigned short lpc_mask;
	volatile unsigned short lpc_trigger;
	volatile unsigned short lpc_level;
	unsigned char pad1[8];

	volatile unsigned short pci_req;	/* offset 0x20 */
	volatile unsigned short pci_mask;
	volatile unsigned short pci_trigger;
	volatile unsigned short pci_level;
	unsigned char pad2[8];

	volatile unsigned short nmi_req;	/* offset 0x30 */
	volatile unsigned short nmi_mask;
	volatile unsigned short nmi_trigger;
	volatile unsigned short nmi_level;
	unsigned char pad3[6];

	volatile unsigned short nmi_redir;	/* offset 0x3E */
	unsigned char pad4[0xBE];

	volatile unsigned short intstatus;	/* offset 0xFE */
};
#endif	/* CONFIG_MIPS_TRIHIDTV_PRO_AV_MIPS */

#endif	/* _MIPS_TRIHIDTVINT_H */
