/*
 *  is_nand.c
 *
 *  Copyright (C) 2000 Steven J. Hill (sjhill@realitydiluted.com)
 *   		  2003 Thomas Gleixner (tglx@linutronix.de)
 *
 * $Id: is_nand.c,v 1.1 2007/09/19 08:43:38 samsheng Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Overview:
 *   This utility writes a binary image directly to a NAND flash
 *   chip or NAND chips contained in DoC devices. This is the
 *   "inverse operation" of nanddump.
 *
 * tglx: Major rewrite to handle bad blocks, write data with or without ECC
 *	 write oob data only on request
 *
 * Bug/ToDo:
 */

#include <ctype.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include "mtd/mtd-user.h"
#define PROGRAM "is_nand"

/*
 * Main program
 */
int main(int argc, char **argv)
{
	int fd;
	struct mtd_info_user meminfo;

	if(argc != 2)
	{
		fprintf(stderr, "Usage: %s <device-file>\n", argv[0]);
		return 1;
	}

	/* Open the device */
	if ((fd = open(argv[1], O_RDWR)) == -1) {
		perror("open flash");
		return 1;
	}

	/* Fill in MTD device capability structure */  
	if (ioctl(fd, MEMGETINFO, &meminfo) != 0) {
		perror("MEMGETINFO");
		close(fd);
		return 1;
	}
	close(fd);
	
	if (meminfo.oobsize != 0)
	{
		return 0;
	}
	return 1;
}

